﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController.GUI;
using GraphController;
using GraphController.Utility;

namespace NTNU.DependencySimulator.GUI
{
    public partial class CycleRefactoringView : RefactoringView
    {
        private List<CycleRefactoringData> _data;

        public CycleRefactoringView()
        {
            InitializeComponent();
        }
        public CycleRefactoringView(List<CycleRefactoringData> l,MainController m)
            : base(m)
        {
            InitializeComponent();
            _data = l;
            PopulateCandidateList();
        }

        protected override void PopulateCandidateList()
        {
            Tracer.WriteTrace("Datagridsource: {0}", _data.Count);
            dataGridView1.AutoGenerateColumns = false;
            dataGridView1.DataSource = _data;
            dataGridView1.Update();
            //dataGridView1.Columns.get


            DataGridViewTextBoxColumn textColumn = new DataGridViewTextBoxColumn();
            textColumn.HeaderText = "Source";
            textColumn.DataPropertyName = "Source";
            dataGridView1.Columns.Add(textColumn);


            DataGridViewTextBoxColumn targetCol = new DataGridViewTextBoxColumn();
            targetCol.DataPropertyName = "Target";
            targetCol.HeaderText = "Target";
            dataGridView1.Columns.Add(targetCol);

            DataGridViewTextBoxColumn crssCol = new DataGridViewTextBoxColumn();
            crssCol.DataPropertyName = "Reachability";
            crssCol.HeaderText = "CRSS";
            dataGridView1.Columns.Add(crssCol); 
            
            DataGridViewTextBoxColumn sccCol = new DataGridViewTextBoxColumn();
            sccCol.DataPropertyName = "SCC";
            sccCol.HeaderText = "SCC";
            dataGridView1.Columns.Add(sccCol);

            DataGridViewTextBoxColumn refactorCol = new DataGridViewTextBoxColumn();
            refactorCol.DataPropertyName = "Strategy";
            refactorCol.HeaderText = "Refactor Strategy";
            dataGridView1.Columns.Add(refactorCol);

            DataGridViewTextBoxColumn reductionCol = new DataGridViewTextBoxColumn();
            reductionCol.DataPropertyName = "ReductionRate";
            reductionCol.HeaderText = "EI Reduction";
            dataGridView1.Columns.Add(reductionCol);

            DataGridViewCheckBoxColumn checkBoxColumn = new DataGridViewCheckBoxColumn();
            checkBoxColumn.HeaderText = "Select";
            checkBoxColumn.DataPropertyName = "IsSelected";
            checkBoxColumn.ReadOnly = false;
            dataGridView1.Columns.Add(checkBoxColumn);
            dataGridView1.Enabled = true;
            dataGridView1.ReadOnly = false;
            dataGridView1.ColumnHeaderMouseDoubleClick += OnColumnHeaderDoubleClick;
        }

        public void OnColumnHeaderDoubleClick(object sender, DataGridViewCellMouseEventArgs a)
        {
            DataGridView v = (DataGridView)sender;

            if (a.ColumnIndex == 6)
            {
                foreach (DataGridViewRow row in dataGridView1.Rows)
                {
                    DataGridViewCheckBoxCell chk = (DataGridViewCheckBoxCell)row.Cells[6];
                    chk.Value = !(chk.Value == null ? false : (bool)chk.Value);
                }
            }
        }
        private void btnApply_Click(object sender, EventArgs e)
        {
            List<CycleRefactoringData> rows = new List<CycleRefactoringData>();
            foreach (DataGridViewRow row in dataGridView1.Rows)
            {
                DataGridViewCheckBoxCell chk = (DataGridViewCheckBoxCell)row.Cells[6];
                try
                {
                    if ((bool)chk.Value == true)
                        rows.Add((CycleRefactoringData)row.DataBoundItem);
                }
                catch(NullReferenceException)
                {
                    continue;
                }
                
            }
            if (rows.Count <= 0)
                MessageBox.Show("No items Selected!");
            else
            {
                try
                {
                    Cursor = Cursors.WaitCursor;
                    _result = _model.SimulateCycleRefactoring(rows);
                    Cursor = Cursors.Default;
                    ShowSimulationResult();
                }
                catch(NullReferenceException er)
                {
                    MessageBox.Show("Error ocurred: \n" + er.StackTrace);
                }
                
            }
        }
    }
}
