﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController;
using GraphController.Utility;

namespace NTNU.DependencySimulator.GUI
{
    public partial class ParameterInputView : Form
    {
        private int topKFanin, topKCrss, topKScc;
        private string mainClass, sorting;
        private MainController model;

        public ParameterInputView()
        {
            InitializeComponent();
        }
        public ParameterInputView(MainController m)
        {
            model = m;
            InitializeComponent();
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            txtBoxCRSS.Text = "" + model.GetTopKCRSS();
            txtBoxFanin.Text = "" + model.GetTopKFanin();
            txtBoxSCC.Text = "" + model.GetTopKSCC();
            Tracer.WriteTrace("mainclass from model: {0}",model.GetMainClass());
            txtBoxMainClass.Text = model.GetMainClass();
            sorting = "";
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if(checkInput())
            {
                try
                {
                    if (!model.ContainsClass(mainClass))
                        throw new ArgumentNullException();

                    model.SetTable1SortingPriority(sorting);
                    model.SetInputParameters(topKFanin, topKCrss, topKScc, mainClass);
                    RefactoringSelection r = new RefactoringSelection(model);
                    r.Show();
                    this.Dispose();
                }
                catch(ArgumentNullException)
                {
                    MessageBox.Show("You have to set the Main class.(You can right click on a node to set it as the main class)");
                }
                
            }

        }
        private bool checkInput()
        {
            try
            {
                topKFanin = Convert.ToInt32(txtBoxFanin.Text);
                if (topKFanin <= 0 || topKFanin > 100) 
                {
                    MessageBox.Show("Fanin Number must be greater than 0 and less than 100");
                    return false;
                }
                topKCrss = Convert.ToInt32(txtBoxCRSS.Text);
                if (topKCrss <= 0 || topKCrss > 100) 
                {
                    MessageBox.Show("CRSS Number must be greater than 0 and less than 100");
                    return false;
                }
                topKScc = Convert.ToInt32(txtBoxSCC.Text);
                if (topKScc <= 0 || topKScc > 100)
                {
                    MessageBox.Show("SCC Number must be greater than 0 and less than 100");
                    return false;
                }
                mainClass = txtBoxMainClass.Text;
                if (radBtnFanin.Checked)
                {
                    sorting = "FANIN";
                }
                else if (radBtnCrss.Checked)
                {
                    sorting = "CRSS";
                }
                else if (radBtnScc.Checked)
                {
                    sorting = "SCC";
                }
                else if (radBtnStatic.Checked)
                {
                    sorting = "STATIC";
                }
                else
                {
                    MessageBox.Show("You have to choose sorting priority.");
                    return false;
                }
            }
            catch (FormatException)
            {
                MessageBox.Show("Input is invalid. Please input a number greater than 0 and less than 100");
                return false;
            }
            catch (OverflowException)
            {
                MessageBox.Show("Something went wrong. Caught OverflowException");
                return false;
            }
            return true;
        }

    }
}
