﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController.Common;
using GraphController;

namespace NTNU.DependencySimulator.GUI
{
    public partial class RemoveReferenceViewer : Form
    {
        private DependencyEdge _edge;
        private MainController _model;
        private Dictionary<string, ReferenceType> _referenceMap;

        public RemoveReferenceViewer()
        {
            InitializeComponent();
        }
        public RemoveReferenceViewer(DependencyEdge e,MainController model)
        {
            InitializeComponent();
            _edge = e;
            _model = model;
            _referenceMap = new Dictionary<string, ReferenceType>();
            groupBox1.Text = "References from " + e.Source.FullName + " -> " + e.Target.FullName;
            InitCheckBoxes();
        }
        private void InitCheckBoxes()
        {
            int yInc = 0;
            foreach(ReferenceType t in GraphData.GetAllTypes())
            {
                CheckBox chk = new CheckBox();

                this.groupBox1.Controls.Add(chk);
                chk.Location = new System.Drawing.Point(7,30 + yInc);
                chk.Size = new System.Drawing.Size(80, 17);
                chk.TabIndex = 0;
                chk.Text = t.ToString();
                chk.UseVisualStyleBackColor = true;
                chk.AutoSize = true;
                chk.Name = t.ToString();
                _referenceMap.Add(t.ToString(), t);
                if (_edge.Types.Contains(t))
                {
                    chk.Checked = true;
                }

                yInc += 24;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            List<ReferenceType> listToAdd = new List<ReferenceType>();
            List<ReferenceType> listToRemove = new List<ReferenceType>();

            foreach(CheckBox chk in this.groupBox1.Controls)
            {
                if (!chk.Checked && _edge.Types.Contains(_referenceMap[chk.Text]))
                {
                    listToRemove.Add(_referenceMap[chk.Text]);
                }
                else if (chk.Checked && !_edge.Types.Contains(_referenceMap[chk.Text]))
                {
                    listToAdd.Add(_referenceMap[chk.Text]);
                }
            }
            _model.PerformEditReferencesInMaps(_edge, listToRemove, listToAdd);
            this.Dispose();
        }
    }
}
