﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController.GUI;
using GraphController.Utility;

namespace NTNU.DependencySimulator.GUI
{
    public partial class ReportViewer : RefactoringView
    {
        private RefactoredEdges _edges;
        private string _sol;
        public ReportViewer(RefactoredEdges edges,string sol)
        {
            _edges = edges;
            _sol = sol;
            InitializeComponent();
            PopulateTable();
        }
        private string WriteReport()
        {
            StringBuilder sb = new StringBuilder();
            string n = System.Environment.NewLine;

            sb.Append("Report" + n + n);
            sb.Append("Elements: " + _edges.Edges.Count + n + n);
            foreach(RefactoredEdge r in _edges.Edges)
            {
                sb.Append(r.Source + "\t" + r.Action + "\t" + r.Target + "\t" + r.Strategy + n);
            }

            return sb.ToString();
        }
        private void PopulateTable()
        {
            Tracer.WriteTrace("Report: {0}",_edges.Edges.Count);
            dataGridView1.AutoGenerateColumns = false;
            dataGridView1.DataSource = _edges.Edges;
            dataGridView1.Update();


            DataGridViewTextBoxColumn sourceColumn = new DataGridViewTextBoxColumn();
            sourceColumn.HeaderText = "Source";
            sourceColumn.DataPropertyName = "Source";
            dataGridView1.Columns.Add(sourceColumn);

            DataGridViewTextBoxColumn actionColumn = new DataGridViewTextBoxColumn();
            actionColumn.HeaderText = "Action";
            actionColumn.DataPropertyName = "Action";
            dataGridView1.Columns.Add(actionColumn);

            DataGridViewTextBoxColumn targetColumn = new DataGridViewTextBoxColumn();
            targetColumn.HeaderText = "Target";
            targetColumn.DataPropertyName = "Target";
            dataGridView1.Columns.Add(targetColumn);

            DataGridViewTextBoxColumn strategyColumn = new DataGridViewTextBoxColumn();
            strategyColumn.HeaderText = "Strategy";
            strategyColumn.DataPropertyName = "Strategy";
            dataGridView1.Columns.Add(strategyColumn);
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void btnWritetoFile_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "txt|*.txt";
            saveFileDialog1.Title = "Save Report";
            saveFileDialog1.ShowDialog();

            // If the file name is not an empty string open it for saving.
            if (saveFileDialog1.FileName != "")
            {
                // Saves the Image via a FileStream created by the OpenFile method.
                //System.IO.FileStream fs = File.OpenWrite(saveFileDialog1.FileName);
                string report = WriteReport();
                File.WriteAllText(saveFileDialog1.FileName, report);
                //byte[] bytes = new byte[report.Length * sizeof(char)];
                //System.Buffer.BlockCopy(report.ToCharArray(), 0, bytes, 0, bytes.Length);
                //fs.Write(bytes,0,0);
                //// Saves the Image in the appropriate ImageFormat based upon the
                //// File type selected in the dialog box.
                //// NOTE that the FilterIndex property is one-based.
                

                //fs.Close();
            }
            this.Dispose();
        }
    }
}
