﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController.GUI;
using GraphController;
using GraphController.Utility;

namespace NTNU.DependencySimulator.GUI
{
    public partial class SimulationResultViewer : Form
    {
        private SimulationResult _result;
        private MainController _model;

        public SimulationResultViewer()
        {
            InitializeComponent();
        }
        public SimulationResultViewer(SimulationResult r,MainController m)
        {
            InitializeComponent();
            this.btnClose.Click += new System.EventHandler(this.btnClose_Click);
            this.btnAddToReport.Click += new System.EventHandler(this.btnAddToReport_Click);
            FormClosed += new FormClosedEventHandler(Close);
    
            _result = r;
            _model = m;
            
            if(_result != null)
                PopulateView(true);
        }
        public SimulationResultViewer(SimulationResult r)
        {
            InitializeComponent();
            this.btnAddToReport.Enabled = false;
            this.btnClose.Enabled = false;

            _result = r;

            PopulateView(false);
        }
        private void PopulateView(bool sim)
        {
            try
            {
                txtBoxMeanBefore.Text = "" + _result.Pre_Mean;
                txtBoxNumCompBefore.Text = "" + _result.Pre_NumberOfNodes;
                txtBoxNumEdgesBefore.Text = "" + _result.Pre_NumberOfEdges;
                txtBoxMaxSCCBefore.Text = "" + _result.Pre_MaxSCC;
                txtBoxStdDevBefore.Text = "" + _result.Pre_StdDev;


                for (int i = 0; i < _result.Post_Mean.Count(); i++ )
                {
                    cmbBoxMeanAfter.Items.Add(_result.Post_Mean.GetValue(i));
                }
                cmbBoxMeanAfter.SelectedIndex = cmbBoxMeanAfter.Items.Count - 1;


                for (int i = 0; i < _result.PostStdDev.Count(); i++)
                {
                    cmbBoxStdDevAfter.Items.Add(_result.PostStdDev.GetValue(i));
                }
                cmbBoxStdDevAfter.SelectedIndex = cmbBoxStdDevAfter.Items.Count - 1;

                txtBoxNumCompAfter.Text = "" + _result.Post_NumberOfNodes;

                txtBoxNumEdgesAfter.Text = "" + _result.Post_NumberOfEdges;

                txtBoxMaxSCCAfter.Text = "" + _result.Post_MaxSCC;
            }
            catch(NullReferenceException)
            {
            }

            gridHistogram.AutoGenerateColumns = false;
            gridHistogram.RowHeadersWidthSizeMode = DataGridViewRowHeadersWidthSizeMode.AutoSizeToAllHeaders;
            gridHistogram.Update();

            DataGridViewTextBoxColumn beforecolumn = new DataGridViewTextBoxColumn();
            beforecolumn.Name = "Before";
            beforecolumn.HeaderText = "Before";
            gridHistogram.Columns.Add(beforecolumn);

            DataGridViewTextBoxColumn aftercolumn = new DataGridViewTextBoxColumn();
            aftercolumn.Name = "After";
            aftercolumn.HeaderText = "After";
            gridHistogram.Columns.Add(aftercolumn);
            

            Tracer.WriteTrace("Simulationresult: \n\tPreHistogram: {0}",_result.GetPreHistogramSize());
            for (int i = 0; i < _result.GetPreHistogramSize(); i++)
            {
                try
                {
                    string range = _result.GetRangeInHistogram(true, i);
                    int freq = _result.GetFrequencyInHistogram(true, i);

                    string range2 = "";
                    int freq2 = 0; 

                    if(sim)
                    {
                        range2 = _result.GetRangeInHistogram(false, i);
                        freq2 = _result.GetFrequencyInHistogram(false, i);
                    }

                    if(sim)
                    {
                        if (_result.GetFrequencyInHistogram(true, i) <= 0 && _result.GetFrequencyInHistogram(false, i) <= 0)
                            continue;
                    }
                    
                    reachHistogram.Series["Before"].Points.AddXY(range, freq);
                    
                    if(sim)
                        reachHistogram.Series["After"].Points.AddXY(range2, freq2);


                    //DataGridViewRow row = (DataGridViewRow)gridHistogram.Rows[0].Clone();
                    //row.HeaderCell.Value = range; 
                    int index = gridHistogram.Rows.Add(freq, freq2);
                    gridHistogram.Rows[index].HeaderCell.Value = range;
                    //row.Cells[0].Value = freq;
                    //row.Cells[1].Value = freq2;
                    //gridHistogram.Rows.Add(row);
                }
                catch(Exception)
                {
                    continue;
                }
                
            }
        }
        private void Close(object sender, FormClosedEventArgs e)
        {
            if (_result.WasAdded)
            {
                return;
            }
            _model.RollbackPreviousResult();
        }
        private void btnClose_Click(object sender, EventArgs e)
        {
            if (_result.WasAdded)
            {
                this.Dispose();
                return;
            }
            _model.RollbackPreviousResult();
            this.Dispose();
        }

        private void btnAddToReport_Click(object sender, EventArgs e)
        {
            _model.AddResult(_result);
            this.Dispose();
        }
    }
}
