﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController.GUI;
using GraphController;
using GraphController.Utility;

namespace NTNU.DependencySimulator.GUI
{
    public partial class SingleComponentRefactoringView : RefactoringView
    {
        private Dictionary<string, DataGridViewColumn> _columnDict;
        private List<SingleComponentRefactoringData> _data;
        private DataGridViewSelectedRowCollection selected;

        public SingleComponentRefactoringView()
        {
            _columnDict = new Dictionary<string, DataGridViewColumn>();
            InitializeComponent();
            PopulateCandidateList();
        }
        public SingleComponentRefactoringView(List<SingleComponentRefactoringData> l,MainController m)
            : base(m)
        {
            _columnDict = new Dictionary<string, DataGridViewColumn>();
            InitializeComponent();
            _data = l;
            PopulateCandidateList();
        }

        class TestObject
        {
            public int OneValue { get; set; }
            public int TwoValue { get; set; }
        }
        protected override void PopulateCandidateList()
        {
            Tracer.WriteTrace("Datagridsource: {0}",_data.Count);
            dataGridView1.AutoGenerateColumns = false;
            dataGridView1.DataSource = _data;
            dataGridView1.Update();
            dataGridView1.CurrentCellDirtyStateChanged += dataGridView1_CurrentCellDirtyStateChanged;
            dataGridView1.CellMouseClick += dataGridView1_CellMouseDown;
            //dataGridView1.Columns.get

            
            DataGridViewTextBoxColumn textColumn = new DataGridViewTextBoxColumn();
            textColumn.HeaderText = "Source";
            textColumn.DataPropertyName = "Source";
            dataGridView1.Columns.Add(textColumn);


            DataGridViewTextBoxColumn targetCol = new DataGridViewTextBoxColumn();
            targetCol.DataPropertyName = "Target";
            targetCol.HeaderText = "Target";
            dataGridView1.Columns.Add(targetCol);

            DataGridViewTextBoxColumn crssCol = new DataGridViewTextBoxColumn();
            crssCol.DataPropertyName = "Reachability";
            crssCol.HeaderText = "Reachability";
            dataGridView1.Columns.Add(crssCol);

            DataGridViewTextBoxColumn faninCol = new DataGridViewTextBoxColumn();
            faninCol.DataPropertyName = "FanIn";
            faninCol.HeaderText = "Fanin";
            dataGridView1.Columns.Add(faninCol);

            DataGridViewTextBoxColumn sccCol = new DataGridViewTextBoxColumn();
            sccCol.DataPropertyName = "SCC";
            sccCol.HeaderText = "SCC";
            dataGridView1.Columns.Add(sccCol);

            DataGridViewTextBoxColumn refactorCol = new DataGridViewTextBoxColumn();
            refactorCol.DataPropertyName = "Strategy";
            refactorCol.HeaderText = "Refactor Strategy";
            dataGridView1.Columns.Add(refactorCol);

            DataGridViewTextBoxColumn reductionCol = new DataGridViewTextBoxColumn();
            reductionCol.DataPropertyName = "ReductionRate";
            reductionCol.HeaderText = "EI Reduction";
            dataGridView1.Columns.Add(reductionCol);

            DataGridViewCheckBoxColumn checkBoxColumn = new DataGridViewCheckBoxColumn();
            DataGridViewColumnHeaderCell hc = new DataGridViewColumnHeaderCell();
            hc.ValueType = typeof(Button);
            
            //checkBoxColumn.HeaderCell = hc;
            checkBoxColumn.HeaderText = "Select";
            checkBoxColumn.DataPropertyName = "IsSelected";
            //checkBoxColumn.Binding = new Binding("Available");
            checkBoxColumn.ReadOnly = false;
            dataGridView1.Columns.Add(checkBoxColumn);

            dataGridView1.Enabled = true;
            dataGridView1.ReadOnly = false;
            dataGridView1.ColumnHeaderMouseDoubleClick += OnColumnHeaderDoubleClick;
            dataGridView1.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
        }
        public void OnColumnHeaderDoubleClick(object sender,DataGridViewCellMouseEventArgs a)
        {
            DataGridView v = (DataGridView)sender;
            
            if(a.ColumnIndex == 7)
            {
                foreach (DataGridViewRow row in dataGridView1.Rows)
                {
                    DataGridViewCheckBoxCell chk = (DataGridViewCheckBoxCell)row.Cells[7];
                    chk.Value = !(chk.Value == null ? false : (bool)chk.Value);
                }
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            List<SingleComponentRefactoringData> rows = new List<SingleComponentRefactoringData>();
            foreach (DataGridViewRow row in dataGridView1.Rows)
            {
                DataGridViewCheckBoxCell chk = (DataGridViewCheckBoxCell)row.Cells[7];
                try
                {
                    if ((bool)chk.Value == true)
                        rows.Add((SingleComponentRefactoringData)row.DataBoundItem);
                }
                catch (NullReferenceException)
                {
                    continue;
                }

            }
            if (rows.Count <= 0)
                MessageBox.Show("No items Selected!");
            else
            {
                Cursor = Cursors.WaitCursor;
                _result = _model.SimulateSingleComponentRefactoring(rows);
                Cursor = Cursors.Default;
                ShowSimulationResult();
            }
        }
        private void dataGridView1_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            DataGridViewCell cell = dgv.CurrentCell;
            if (cell.RowIndex >= 0 && cell.ColumnIndex == 7) // My checkbox column
            {
                // If checkbox value changed, copy it's value to all selectedrows
                bool checkvalue = false;
                if (dgv.Rows[cell.RowIndex].Cells[cell.ColumnIndex].EditedFormattedValue != null && dgv.Rows[cell.RowIndex].Cells[cell.ColumnIndex].EditedFormattedValue.Equals(true))
                    checkvalue = true;

                if(selected == null)
                    return;

                for (int i = 0; i < selected.Count; i++)
                    dgv.Rows[selected[i].Index].Cells[cell.ColumnIndex].Value = checkvalue;

                selected = null;
            }

            dataGridView1.CommitEdit(DataGridViewDataErrorContexts.Commit);
        }

        private void dataGridView1_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            if(dataGridView1.SelectedRows.Count > 1)
                selected = dataGridView1.SelectedRows;
        }
    }
}
