﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using GraphController.GUI;
using GraphController;
using GraphController.Utility;

namespace NTNU.DependencySimulator.GUI
{
    public partial class SystemRefactoringView : RefactoringView
    {
        private List<SystemRefactoringData> _data;
        public SystemRefactoringView()
        {
            InitializeComponent();
        }
        public SystemRefactoringView(List<SystemRefactoringData> l,MainController m)
            : base(m)
        {
            InitializeComponent();
            _data = l;
            PopulateCandidateList();
        }

        protected override void PopulateCandidateList()
        {
            Tracer.WriteTrace("Datagridsource: {0}", _data.Count);
            dataGridView1.AutoGenerateColumns = false;
            dataGridView1.DataSource = _data;
            dataGridView1.Update();
            //dataGridView1.Columns.get


            DataGridViewTextBoxColumn textColumn = new DataGridViewTextBoxColumn();
            textColumn.HeaderText = "Class";
            textColumn.DataPropertyName = "Class";
            dataGridView1.Columns.Add(textColumn);

            DataGridViewTextBoxColumn crssCol = new DataGridViewTextBoxColumn();
            crssCol.DataPropertyName = "Reachability";
            crssCol.HeaderText = "Reachability";
            dataGridView1.Columns.Add(crssCol);

            DataGridViewTextBoxColumn faninCol = new DataGridViewTextBoxColumn();
            faninCol.DataPropertyName = "Fanin";
            faninCol.HeaderText = "Fanin";
            dataGridView1.Columns.Add(faninCol);

            DataGridViewTextBoxColumn sccCol = new DataGridViewTextBoxColumn();
            sccCol.DataPropertyName = "SCC";
            sccCol.HeaderText = "SCC";
            dataGridView1.Columns.Add(sccCol);

            DataGridViewTextBoxColumn refactorCol = new DataGridViewTextBoxColumn();
            refactorCol.DataPropertyName = "Strategy";
            refactorCol.HeaderText = "Refactor Strategy";
            dataGridView1.Columns.Add(refactorCol);

            DataGridViewTextBoxColumn reductionCol = new DataGridViewTextBoxColumn();
            reductionCol.DataPropertyName = "ReductionRate";
            reductionCol.HeaderText = "EI Reduction";
            dataGridView1.Columns.Add(reductionCol);
        }


        private void btnApply_Click(object sender, EventArgs e)
        {
            int num = 0;
            try
            {
                num = int.Parse(txtBoxRefatorings.Text);
            }
            catch(FormatException)
            {
                MessageBox.Show("Please insert number of refactorings.");
                return;
            }
                 
            if (num > 0)
            {
                Cursor = Cursors.WaitCursor;
                _result = _model.SimulateSystemRefactoring(num);
                Cursor = Cursors.Default;
                ShowSimulationResult();
            }
            else
                MessageBox.Show("Number must be higher than zero.");
            
        }
    }
}
