﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Windows.Controls;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;
using QuickGraph;
using GraphController.Utility;
using java.util;
using GraphController.Common;
using melton.hayden.jepends.analysis;
using tosin.oyetoyan.research.main.graph;
using tosin.oyetoyan.research.main.utility;
using ntnu.se.research.eades;

namespace GraphController
{
    public class DependencyGraph : BidirectionalGraph<DependencyVertex, DependencyEdge>
    {
        public DependencyGraph()
            : base()
        {
        }

        public void AddEdgeToGraph(DependencyVertex s, DependencyVertex t, ReferenceType type,bool pub)
        {
            DependencyEdge e;
            if ((e = CustomContainsEdge(s, t)) != null)
            {
                DependencyEdge e2;
                if ((e2 = CustomContainsEdge(t, s)) != null)
                {
                    e.IsCyclic = true;
                    e2.IsCyclic = true;
                }

                //Tracer.WriteTrace("Edge between {0} and {1} already exists, adding {2}", s.FullName, t.FullName, r);
                e.AddReferenceType(type);
                //AddEdgeToGraph(e);
            }
            else
            {
                //Tracer.WriteTrace("new Edge between {0} and {1} , adding {2}", s.Name, t.Name, r);
                e = new DependencyEdge(s, t, type);
                e.Reference.IsPublished = pub;
                AddEdgeToGraph(e);
            }
        }
        public void AddEdge(string name,string reference,ReferenceType t,bool pub)
        {
            DependencyVertex so = Vertices.Where(n => n.FullName == name).First();
            DependencyVertex ta = Vertices.Where(n => n.FullName == reference).First();

            if (so == null || ta == null)
                return;
            AddEdgeToGraph(so, ta, t, pub);
        }
        public void AddEdgeToGraph(DependencyEdge e)
        {
            DependencyEdge edge = e;

            if (ContainsVertex(e.Source))
            {
                if (ContainsVertex(e.Target))
                {
                    this.AddEdge(edge);
                    edge.Source.AddReference(edge);
                }
                else
                {
                    edge = CalculateEdgeInMap(e.Source, null, e);
                    this.AddEdge(edge);
                    edge.Source.AddReference(edge);
                }

            }
            else if (ContainsVertex(e.Target))
            {
                edge = CalculateEdgeInMap(null, e.Target, e);
                this.AddEdge(edge);
                edge.Source.AddReference(edge);
            }
            else
            {
                edge = CalculateEdgeInMap(null, null, e);
                this.AddEdge(edge);
                edge.Source.AddReference(edge);
            } 

            DependencyEdge e2;
            if ((e2 = CustomContainsEdge(edge.Target, edge.Source)) != null)
            {
                edge.IsCyclic = true;
                edge.CyclicSibling = e2;
                e2.IsCyclic = true;
                e2.CyclicSibling = edge;
            } 
           
        }
        public DependencyVertex GetVertex(string name)
        {
            return Vertices.Where(n => n.FullName == name).First();
        }
        public void RemoveDependency(string s,string t,ReferenceType type)
        {
            DependencyEdge e = Edges.Where(edge => edge.Source.FullName == s && edge.Target.FullName == t).First();
            if(type == ReferenceType.DEPENDENCY)
            {
                RemoveEdge(e);
            }
            else
            {
                e.Reference.RemoveReference(type);
            }
            //e.Reference.RemoveReference(type);
        }
        private DependencyEdge CustomContainsEdge(DependencyVertex s, DependencyVertex t)
        {
            var list = Edges.Where(n => n.Source == s && n.Target == t);
            if (list.Count() > 0)
                return list.First();
            else
                return null;
        }
        public string GetEdgeInformationForVertex(DependencyVertex v)
        {
            
            var list = Edges.Where(e => e.Source.FullName == v.FullName || e.Target.FullName == v.FullName);
            if (list.Count() <= 0)
                return null;
            StringBuilder sb = new StringBuilder();
            foreach(DependencyEdge e in list)
            {
                sb.Append(e.Source.FullName + "\t->\t" + e.Target.FullName + ": " + e.Reference.Tooltip + System.Environment.NewLine);
            }
            return sb.ToString();
        }
        
        public DependencyVertex TryRemoveNode(DependencyVertex v)
        {
            try
            {
                DependencyVertex ve = FindVertexInMap(v.FullName);
                return ve;
            }
            catch(InvalidOperationException)
            {
                return null;
            }
        }
        public void RemoveVertex(string name)
        {
            DependencyVertex v = Vertices.Where(n => n.FullName == name).First();
            if(ContainsVertex(v))
                RemoveVertex(v);
        }
        public void RemoveEdgeFromGraph(DependencyEdge e)
        {
            DependencyEdge edge = e;

            try
            {
                if (ContainsVertex(e.Source))
                {
                    if (ContainsVertex(e.Target))
                    {
                        this.RemoveEdge(edge);
                        edge.Source.RemoveReference(edge);
                    }
                    else
                    {
                        edge = CalculateEdgeInMap(e.Source, null, e);
                        this.RemoveEdge(edge);
                        edge.Source.RemoveReference(edge);
                    }

                }
                else if (ContainsVertex(e.Target))
                {
                    edge = CalculateEdgeInMap(null, e.Target, e);
                    this.RemoveEdge(edge);
                    edge.Source.RemoveReference(edge);
                }
                else
                {
                    edge = CalculateEdgeInMap(null, null, e);
                    this.RemoveEdge(edge);
                    edge.Source.RemoveReference(edge);
                } 
            }
            catch(InvalidOperationException)
            {
                return;
            }
            

            DependencyEdge e2;
            if ((e2 = CustomContainsEdge(edge.Target, edge.Source)) != null)
            {
                edge.IsCyclic = false;
                edge.CyclicSibling = null;
                e2.IsCyclic = false;
                e2.CyclicSibling = null;
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="source"></param>
        /// <param name="target"></param>
        /// <param name="edge"></param>
        /// <exception cref="InvalidOperationException"></exception>
        /// <returns></returns>
        private DependencyEdge CalculateEdgeInMap(DependencyVertex source, DependencyVertex target, DependencyEdge edge)
        {
            if(source == null)
            {
                source = FindVertexInMap(edge.Source.FullName);
            }
            if(target == null)
            {
                target = FindVertexInMap(edge.Target.FullName);
            }

            return new DependencyEdge(source, target, edge.Types);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="fullname"></param>
        /// <exception cref="InvalidOperationException"></exception>
        /// <returns></returns>
        private DependencyVertex FindVertexInMap(string fullname)
        {
            DependencyVertex vertex = Vertices.Where(ed => ed.FullName == fullname).First();
            return vertex;
        }
    }
   
    
    public class MyGraphLayout : GraphSharp.Controls.GraphLayout<DependencyVertex, DependencyEdge, DependencyGraph>
    {
        private bool shouldLayout = true;

        public MyGraphLayout()
            : base()
        {
            
            //IsAnimationEnabled = false;
            MoveAnimation = null;
        }
        
        protected override bool CanLayout
        {
            get { return shouldLayout; }
        }

        public bool ShouldLayout
        {
            get { return shouldLayout; }
            set { shouldLayout = value; }
        }

    }
    
}
