﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using java.util;
using GraphController.Utility;
using GraphController.Logic;
using tosin.oyetoyan.research.main.utility;

namespace GraphController.Common
{
    public class MapModel
    {
        private Map _all;
        private Map _publishedtypes; //Return type or parameter type in a public or protected
        private Map _memberfieldusage; //Variables inside 
        private Map _staticFieldUsage;
        private Map _methodTypes; //Static or non-static
        private Map _methodInvocations;  //method invocation
        private Map _staticmethodInvocations; //static method invocation
        private Map _constructorinvocations;
        private Map _superTypes; //Could be removed in the future
        private Map _basetypes;
        private Map _interfacetypes; 
        private Map _methodsize;
        private Map _classtypes; //I for interface, A for abstract or C for class

        private GraphActions _currentActions;
        private ReferenceType _currentReference;
        public event EventHandler<MapEventArgs> ThrowMapChanged;

        public MapModel()
        {
            _all = new HashMap();
            _publishedtypes = new HashMap();
            _memberfieldusage = new HashMap();
            _staticFieldUsage = new HashMap();
            _methodInvocations = new HashMap();
            _staticmethodInvocations = new HashMap();
            _methodTypes = new HashMap();
            _constructorinvocations = new HashMap();
            _basetypes = new HashMap();
            _interfacetypes = new HashMap();
            _superTypes = new HashMap();
            _methodsize = new HashMap();
            _classtypes = new HashMap();
            _currentActions = new GraphActions();
        }
        public MapModel(MapModel m)
        {
            _all = CreateMap.copy(m._all);
            _basetypes = CreateMap.copy(m._basetypes);
            _publishedtypes = CreateMap.copy(m._publishedtypes);
            _memberfieldusage = CreateMap.copy(m._memberfieldusage);
            _staticFieldUsage = CreateMap.copy(m._staticFieldUsage);
            _methodInvocations = CreateMap.copy(m._methodInvocations);
            _staticmethodInvocations = CreateMap.copy(m._staticmethodInvocations);
            _methodTypes = CreateMap.copy(m._methodTypes);
            _constructorinvocations = CreateMap.copy(m._constructorinvocations);
            _interfacetypes = CreateMap.copy(m._interfacetypes);
            _superTypes = CreateMap.copy(m._superTypes);
            _methodsize = CreateMap.copy(m._methodsize);
            _classtypes = CreateMap.copy(m._classtypes);
        }


        public Map AllDependencies
        {
            get { return _all; }
            set { _all = value; }
        }

        public Map PublishedTypes
        {
            get { return _publishedtypes; }
            set { _publishedtypes = value; }
        }
        public Map ClassTypes
        {
            get { return _classtypes; }
            set { _classtypes = value; }
        }

        public Map MethodTypes
        {
            get { return _methodTypes; }
            set { _methodTypes = value; }
        }
        public Map Memberfields
        {
            get { return _memberfieldusage; }
            set { _memberfieldusage = value; }
        }
        public Map StaticMemberUsage
        {
            get { return _staticFieldUsage; }
            set { _staticFieldUsage = value; }
        }

        public Map MethodInvocations
        {
            get { return _methodInvocations; }
            set { _methodInvocations = value; }
        }

        public Map StaticMethodInvocations
        {
            get { return _staticmethodInvocations; }
            set { _staticmethodInvocations = value; }
        }

        public Map Constructorinvocations
        {
            get { return _constructorinvocations; }
            set { _constructorinvocations = value; }
        }
        public Map SuperTypes
        {
            get { return _superTypes; }
        }
        public Map Basetypes
        {
            get { return _basetypes; }
            set { _basetypes = value; }
        }

        public Map Interfacetypes
        {
            get { return _interfacetypes; }
            set { _interfacetypes = value; }
        }
        public Map MethodSize
        {
            get { return _methodsize; }
            set { _methodsize = value; }
        }

        public void AddClass(string name)
        {
            _all.put(name, new HashSet());
            _publishedtypes.put(name, new HashSet());
            _memberfieldusage.put(name, new HashSet());
            _staticFieldUsage.put(name, new HashSet());
            _methodInvocations.put(name, new HashSet());
            _methodTypes.put(name, new HashSet());
            _staticmethodInvocations.put(name, new HashSet());
            _constructorinvocations.put(name, new HashSet());
            _basetypes.put(name, new HashSet());
            _interfacetypes.put(name, new HashSet());
            _superTypes.put(name, new HashSet());
            _methodsize.put(name, new HashSet());
            _classtypes.put(name, new HashSet());

            MapEventArgs e = new MapEventArgs(name, null, ReferenceType.NODE, true);
            FireMapEvent(e);
        }
        public void RemoveClass(string name)
        {
            _all.remove(name);
            _publishedtypes.remove(name);
            _memberfieldusage.remove(name);
            _methodInvocations.remove(name);
            _methodTypes.remove(name);
            _staticmethodInvocations.remove(name);
            _constructorinvocations.remove(name);
            _basetypes.remove(name);
            _interfacetypes.remove(name);
            _superTypes.remove(name);
            _methodsize.remove(name);
            _classtypes.remove(name);

            MapEventArgs e = new MapEventArgs(name, null, ReferenceType.NODE, false);
            FireMapEvent(e);
        }
        public void AddDependency(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_all.get(name);
            if( s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, true);
                FireMapEvent(e);
            }
            
        }
        public void RemoveDependency(string name,string reference)
        {
            Set s = (Set)_all.get(name);
            if(s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.DEPENDENCY, false);
                FireMapEvent(e);
            }
        }
        public void AddPublishedType(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_publishedtypes.get(name);
            if( s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, true, true);
                FireMapEvent(e);
            }
        }
        public void RemovePublishedType(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_publishedtypes.get(name);
            if( s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, false, true);
                FireMapEvent(e);
            }
        }
        public void AddMemberField(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_memberfieldusage.get(name);
            if( s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, true);
                FireMapEvent(e);
            }
        }
        public void RemoveMemberField(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_memberfieldusage.get(name);
            if(s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, false);
                FireMapEvent(e);
            }
        }
        public void AddStaticFieldUsage(string name, string reference)
        {
            Set s = (Set)_staticFieldUsage.get(name);
            if (s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.STATICFIELD, true);
                FireMapEvent(e);
            }
        }
        public void RemoveStaticFieldUsage(string name, string reference)
        {
            Set s = (Set)_staticFieldUsage.get(name);
            if (s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.STATICFIELD, false);
                FireMapEvent(e);
            }
        }
        public void AddMethodInvocation(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_methodInvocations.get(name);
            if (s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, true);
                FireMapEvent(e);
            }
        }
        public void RemoveMethodInvocation(string name, string reference,ReferenceType t)
        {
            Set s = (Set)_methodInvocations.get(name);
            if (s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, t, false);
                FireMapEvent(e);
            }
        }
        public void AddStaticMethodInvocation(string name, string reference)
        {
            Set s = (Set)_staticmethodInvocations.get(name);
            if (s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.STATICMETHODINVOCATION, true);
                FireMapEvent(e);
            }
        }
        public void RemoveStaticMethodInvocation(string name, string reference)
        {
            Set s = (Set)_staticmethodInvocations.get(name);
            if (s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.STATICMETHODINVOCATION, false);
                FireMapEvent(e);
            }
        }
        public void AddMethodTypes(string name, bool _static, bool _nonStatic)
        {
            bool flag = false;
            Set s = (Set)_methodTypes.get(name);
            if(_static)
                flag = s.add("static");
            if (_nonStatic)
                flag = s.add("nonstatic");

            //if(flag)
            //{
            //    MapEventArgs e = new MapEventArgs(name, null, _currentReference, true);
            //    FireMapEvent(e);
            //}
        }
        public void RemoveMethodTypes(string name, bool _static, bool _nonStatic)
        {
            bool flag = false;
            Set s = (Set)_methodTypes.get(name);
            if (_static)
                flag = s.remove("Static");
            if (_nonStatic)
                flag = s.remove("Nonstatic");

            //if (flag)
            //{
            //    MapEventArgs e = new MapEventArgs(name, null, _currentReference, false);
            //    FireMapEvent(e);
            //}
        }
        public void AddConstructorInvocation(string name, string reference)
        {
            Set s = (Set)_constructorinvocations.get(name);
            if (s.add(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.CONSTRUCTORINVOCATION, true);
                FireMapEvent(e);
            }
        }
        public void RemoveConstructorInvocation(string name, string reference)
        {
            Set s = (Set)_constructorinvocations.get(name);
            if (s.remove(reference))
            {
                MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.CONSTRUCTORINVOCATION, false);
                FireMapEvent(e);
            }
        }
        public void AddBaseType(string name, string reference)
        {
            Set s = (Set)_superTypes.get(name);
            if (s.add(reference))
            {
                Set s2 = (Set)_basetypes.get(name);
                if (s2.add(reference))
                {
                    MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.BASE, true);
                    FireMapEvent(e);
                }
                else
                {
                    s.remove(reference);
                }
            }
        }
        public void RemoveBaseType(string name, string reference)
        {
            Set s = (Set)_superTypes.get(name);
            if (s.remove(reference))
            {
                Set s2 = (Set)_basetypes.get(name);
                if (s2.remove(reference))
                {
                    MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.BASE, false);
                    FireMapEvent(e);
                    
                }
                else
                {
                    s.add(reference);
                }
            }
            
        }
        public void AddInterface(string name, string reference)
        {
            Set s = (Set)_superTypes.get(name);
            if (s.add(reference))
            {
                Set s2 = (Set)_interfacetypes.get(name);
                if (s2.add(reference))
                {
                    MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.INTERFACE, true);
                    FireMapEvent(e);
                }
                else
                {
                    s.remove(reference);
                }
            }
        }
        public void RemoveInterface(string name, string reference)
        {
            Set s = (Set)_superTypes.get(name);
            if (s.remove(reference))
            {
                Set s2 = (Set)_interfacetypes.get(name);
                if (s2.remove(reference))
                {
                    MapEventArgs e = new MapEventArgs(name, reference, ReferenceType.INTERFACE, false);
                    FireMapEvent(e);
                }
                else
                {
                    s.add(reference);
                }
            }
        }
        public void AddMethodSize(string name, string size)
        {
            Set s = (Set)_methodsize.get(name);
            if (s.add(size))
            {
                //MapEventArgs e = new MapEventArgs(name, size, _currentReference, true);
                //FireMapEvent(e);
            }
        }
        public void AddClassType(string name,string type)
        {
            Set s = (Set)_classtypes.get(name);
            if (s.add(type))
            {
                //MapEventArgs e = new MapEventArgs(name, type, _currentReference, true);
                //FireMapEvent(e);
            }
        }
        public void RemoveClassType(string name, string type)
        {
            Set s = (Set)_classtypes.get(name);
            if (s.remove(type))
            {
                //MapEventArgs e = new MapEventArgs(name, type, _currentReference, false);
                //FireMapEvent(e);
            }
        }
        public void WriteMapsToFiles(string dir)
        {
            if (!Directory.Exists(dir))
                Directory.CreateDirectory(dir);

            System.IO.File.WriteAllText(dir + "\\All.txt", MapUtility.WriteMapToString(_all));
            System.IO.File.WriteAllText(dir + "\\Published.txt", MapUtility.WriteMapToString(_publishedtypes));
            System.IO.File.WriteAllText(dir + "\\StaticFieldUsage.txt", MapUtility.WriteMapToString(_staticFieldUsage));
            System.IO.File.WriteAllText(dir + "\\StaticMethods.txt", MapUtility.WriteMapToString(_staticmethodInvocations));
            System.IO.File.WriteAllText(dir + "\\Interfaces.txt", MapUtility.WriteMapToString(_interfacetypes));
            System.IO.File.WriteAllText(dir + "\\ConstructorInvocations.txt", MapUtility.WriteMapToString(_constructorinvocations));
            System.IO.File.WriteAllText(dir + "\\MemberFieldUsage.txt", MapUtility.WriteMapToString(_memberfieldusage));
            System.IO.File.WriteAllText(dir + "\\Methods.txt", MapUtility.WriteMapToString(_methodInvocations));
            System.IO.File.WriteAllText(dir + "\\MethodSize.txt", MapUtility.WriteMapToString(_methodsize));
            System.IO.File.WriteAllText(dir + "\\Base.txt", MapUtility.WriteMapToString(_basetypes));
            System.IO.File.WriteAllText(dir + "\\MethodTypes.txt", MapUtility.WriteMapToString(_methodTypes));
            System.IO.File.WriteAllText(dir + "\\SuperTypes.txt", MapUtility.WriteMapToString(_superTypes));
            System.IO.File.WriteAllText(dir + "\\ClassTypes.txt", MapUtility.WriteMapToString(_classtypes));
        }
        
        public void Equals(MapModel m)
        {
            Tracer.WriteTrace("Checking All: "); MapUtility.MapEqual(_all, m._all);
            Tracer.WriteTrace("Checking Base: "); MapUtility.MapEqual(_basetypes, m._basetypes);
            Tracer.WriteTrace("Checking Classtypes: "); MapUtility.MapEqual(_classtypes, m._classtypes);
            Tracer.WriteTrace("Checking Constructors: "); MapUtility.MapEqual(_constructorinvocations, m._constructorinvocations);
            Tracer.WriteTrace("Checking Interface: "); MapUtility.MapEqual(_interfacetypes, m._interfacetypes);
            Tracer.WriteTrace("Checking FieldUsage: "); MapUtility.MapEqual(_memberfieldusage, m._memberfieldusage);
            Tracer.WriteTrace("Checking MethodInvoke: "); MapUtility.MapEqual(_methodInvocations, m._methodInvocations);
            Tracer.WriteTrace("Checking MethodSize: "); MapUtility.MapEqual(_methodsize, m._methodsize);
            Tracer.WriteTrace("Checking MethodTypes: "); MapUtility.MapEqual(_methodTypes, m._methodTypes);
            Tracer.WriteTrace("Checking Published: "); MapUtility.MapEqual(_publishedtypes, m._publishedtypes);
            Tracer.WriteTrace("Checking StaticFieldUsage: "); MapUtility.MapEqual(_staticFieldUsage, m._staticFieldUsage);
            Tracer.WriteTrace("Checking StaticMethod: "); MapUtility.MapEqual(_staticmethodInvocations, m._staticmethodInvocations);
            Tracer.WriteTrace("Checking SuperTypes: "); MapUtility.MapEqual(_superTypes, m._superTypes);
        }
        
        public void PrintMaps()
        {
            MapUtility.PrintMap(_all, "All");
            MapUtility.PrintMap(_basetypes, "Base");
            MapUtility.PrintMap(_classtypes, "Classtypes");
            MapUtility.PrintMap(_constructorinvocations, "Constructors");
            MapUtility.PrintMap(_interfacetypes, "Interfaces");
            MapUtility.PrintMap(_memberfieldusage, "MemberFields");
            MapUtility.PrintMap(_methodInvocations, "MethodInvocations");
            MapUtility.PrintMap(_methodsize, "MethodSize");
            MapUtility.PrintMap(_methodTypes, "MethodTypes");
            MapUtility.PrintMap(_publishedtypes, "Published");
            //PrintMap(_scc, "Scc");
            MapUtility.PrintMap(_staticmethodInvocations, "StaticMethods");
            MapUtility.PrintMap(_superTypes, "Supertypes");
        }
        public void UndoAction()
        {
            Tracer.WriteTrace("UndoAction()");
            if (_currentActions.CurrentActions.Count <= 0)
                return;
            GraphAction actionToUndo = _currentActions.GetLastCurrentAction();

            switch (actionToUndo.Type)
            {
                case GraphActionType.ADDEDGE:
                    RemoveEdgeAction(actionToUndo);
                    break;
                case GraphActionType.ADDNODE:
                    RemoveNodeAction(actionToUndo);
                    break;
                case GraphActionType.REMOVEEDGE:
                    AddEdgeAction(actionToUndo);
                    break;
                case GraphActionType.REMOVENODE:
                    AddNodeAction(actionToUndo);
                    break;
                case GraphActionType.EDITREFERENCES:
                    RedoEditReferencesInMaps(actionToUndo);
                    break;
                default:
                    return;
            }
            _currentActions.AddRemovedAction(actionToUndo);

        }
        public void RedoAction()
        {
            if (_currentActions.RemovedActions.Count <= 0)
                return;

            GraphAction actionToRedo = _currentActions.GetFirstRemovedAction();
            PerformAction(actionToRedo, true);
            _currentActions.RedoAction(actionToRedo);
        }
        public void PerformAction(GraphAction a, bool redo)
        {
            switch (a.Type)
            {
                case GraphActionType.ADDEDGE:
                    AddEdgeAction(a);
                    break;
                case GraphActionType.ADDNODE:
                    AddNodeAction(a);
                    break;
                case GraphActionType.REMOVEEDGE:
                    RemoveEdgeAction(a);
                    break;
                case GraphActionType.REMOVENODE:
                    RemoveNodeAction(a);
                    break;
                case GraphActionType.EDITREFERENCES:
                    EditReferencesInMaps(a);
                    break;
                default:
                    return;
            }
            _currentActions.AddCurrentAction(a);
            if (!redo) 
                _currentActions.AddNewAction(a);
            //Tracer.WriteTrace("PerformAction() success");
            //PrintActions();
        }
        private void AddEdgeAction(GraphAction a)
        {
            AddEdgeToMaps(a.SingleEdge);
        }

        private void RemoveEdgeAction(GraphAction a)
        {
            RemoveEdgeFromMaps(a.SingleEdge);
        }
        private void AddNodeAction(GraphAction a)
        {
            //Tracer.WriteTrace("AddNodeAction(), node: {0} contains: {1}", a.Node.FullName, _graphToVisualize.ContainsVertex(a.Node));
            AddNodeToMaps(a.Node);
            if(a.Edges != null)
            {
                foreach (DependencyEdge edge in a.Edges)
                {
                    AddEdgeToMaps(edge);
                }
            }
            
            //if (!_graphToVisualize.ContainsVertex(a.Node))
            //{

            //    _graphToVisualize.AddVertex(a.Node);
            //    Tracer.WriteTrace("After AddVertex, node: {0} contains: {1}", a.Node.FullName, _graphToVisualize.ContainsVertex(a.Node));

            //    _currentCollection.Add(a.Node);
            //    if (a.Edges != null)
            //    {
            //        foreach (MyEdgeBase edge in a.Edges)
            //        {
            //            if (!AddEdge(edge))
            //                return false;
            //            //else
            //            //    if (!AddEdgeToMaps(edge))
            //            //        return false;
            //        }
            //        Tracer.WriteTrace("AddNode{0} : return true", a.Node);
            //        return true;
            //    }
            //    else
            //    {
            //        Tracer.WriteTrace("AddNode{0} : return true", a.Node);
            //        return true;
            //    }

            //}
            //Tracer.WriteTrace("RemoveNode{0} : return false", a.Node);
            //return false;
        }
        private void RemoveNodeAction(GraphAction a)
        {
            //Tracer.WriteTrace("RemoveNodeAction(), node: {0} contains: {1}", a.Node.FullName, _graphToVisualize.ContainsVertex(a.Node));
            if (a.Edges != null)
            {
                foreach (DependencyEdge e in a.Edges)
                {
                    RemoveEdgeFromMaps(e);
                }
            }
            RemoveNodeFromMaps(a.Node);
            //if (_graphToVisualize.ContainsVertex(a.Node))
            //{
            //    RemoveEdgesInNode(a);
            //    return true;
            //}
            //else
            //{
            //    MyVertexBase temp;
            //    if((temp = _graphToVisualize.TryRemoveNode(a.Node)) != null)
            //    {
            //        a.Node = temp;
            //        RemoveEdgesInNode(a);
            //        return true;
            //    }
            //}
            //Tracer.WriteTrace("RemoveNode: {0} : return false", a.Node);
            //return false;
        }
        private void RemoveNodeFromMaps(DependencyVertex v)
        {
            RemoveClass(v.FullName);
        }
        private void AddNodeToMaps(DependencyVertex v)
        {
            AddClass(v.FullName);
        }
        private void AddEdgeToMaps(DependencyEdge e)
        {
            List<ReferenceType> types = new List<ReferenceType>(e.Types);
            foreach (ReferenceType t in types)
            {
                AddReferenceToMaps(e.Reference, t);
            }
            AddDependency(e.Source.FullName, e.Target.FullName,ReferenceType.DEPENDENCY);
        }
        private void RemoveEdgeFromMaps(DependencyEdge e)
        {
            List<ReferenceType> types = new List<ReferenceType>(e.Types);
            foreach (ReferenceType t in types)
            {
                RemoveReferenceFromMaps(e.Reference, t);
            }
            
            RemoveDependency(e.Source.FullName, e.Target.FullName);
        }
        private void AddReferenceToMaps(GraphReference r, ReferenceType t)
        {
            DependencyVertex source = r.Source;
            DependencyVertex target = r.Target;
            _currentReference = t;

            switch (t)
            {
                case ReferenceType.ADDEDINGUI:
                    AddDependency(source.FullName, target.FullName,ReferenceType.ADDEDINGUI);
                    break;
                case ReferenceType.BASE:
                    AddBaseType(source.FullName, target.FullName);
                    break;
                case ReferenceType.INTERFACE:
                    AddInterface(source.FullName, target.FullName);
                    break;
                case ReferenceType.MEMBERFIELD:
                    AddMemberField(source.FullName, target.FullName,ReferenceType.MEMBERFIELD);
                    break;
                case ReferenceType.MEMBERREFERENCE:
                    AddMemberField(source.FullName, target.FullName, ReferenceType.MEMBERREFERENCE);
                    break;
                case ReferenceType.STATICMEMBERREFERENCE:
                    AddStaticFieldUsage(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODBODY:
                    //_mapModel.AddMethod(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODPARAMETER:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODPARAMETER);
                    break;
                case ReferenceType.METHODPARAMETERPUB:
                    AddPublishedType(source.FullName, target.FullName, ReferenceType.METHODPARAMETERPUB);
                    break;
                case ReferenceType.METHODRETURNTYPE:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODRETURNTYPE);
                    break;
                case ReferenceType.METHODRETURNTYPEPUB:
                    AddPublishedType(source.FullName, target.FullName, ReferenceType.METHODRETURNTYPEPUB);
                    break;
                case ReferenceType.STATICFIELD:
                    AddMemberField(source.FullName, target.FullName,ReferenceType.STATICFIELD);
                    break;
                case ReferenceType.CONSTRUCTORINVOCATION:
                    AddConstructorInvocation(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODINVOCATION:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODINVOCATION);
                    break;
                case ReferenceType.STATICMETHODINVOCATION:
                    AddStaticMethodInvocation(source.FullName, target.FullName);
                    break;
                case ReferenceType.MEMBERPROPERTY:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.MEMBERPROPERTY);
                    break;
                default:
                    return;
            }
            r.AddReferenceType(t);
        }
        
        private bool RedoEditReferencesInMaps(GraphAction a)
        {
            bool flag = true;
            if (a.ToAdd.Count > 0)
                flag = RemoveReferencesFromMaps(a.SingleEdge, a.ToAdd);
            if (a.ToRemove.Count > 0)
                flag = AddReferencesToMaps(a.SingleEdge, a.ToRemove);

            return flag;
        }
        private bool EditReferencesInMaps(GraphAction a)
        {
            bool flag = true;
            if (a.ToRemove.Count > 0)
                flag = RemoveReferencesFromMaps(a.SingleEdge, a.ToRemove);
            if (a.ToAdd.Count > 0)
                flag = AddReferencesToMaps(a.SingleEdge, a.ToAdd);

            return flag;
        }
        private bool AddReferencesToMaps(DependencyEdge e, List<ReferenceType> types)
        {
            bool flag = true;
            if (e != null && types != null)
            {
                foreach (ReferenceType t in types)
                {
                    GraphReference r = e.Reference;
                    AddReferenceToMaps(r, t);
                }
            }

            return flag;
        }
        private bool RemoveReferencesFromMaps(DependencyEdge e, List<ReferenceType> types)
        {
            bool flag = true;
            if (e != null && types != null)
            {
                foreach (ReferenceType t in types)
                {
                    GraphReference r = e.Reference;
                    RemoveReferenceFromMaps(r, t);
                    
                        //if (!r.RemoveReference(t))
                        //    flag = false;

                    
                }
            }

            return flag;
        }
        private void RemoveReferenceFromMaps(GraphReference r, ReferenceType t)
        {
            DependencyVertex source = r.Source;
            DependencyVertex target = r.Target;
            _currentReference = t;
            
            switch (t)
            {
                case ReferenceType.BASE:
                    RemoveBaseType(source.FullName, target.FullName);
                    break;
                case ReferenceType.INTERFACE:
                    RemoveInterface(source.FullName, target.FullName);
                    break;
                case ReferenceType.MEMBERFIELD:
                    RemoveMemberField(source.FullName, target.FullName,t);
                    break;
                case ReferenceType.MEMBERREFERENCE:
                    RemoveMemberField(source.FullName, target.FullName,t);
                    break;
                case ReferenceType.STATICMEMBERREFERENCE:
                    RemoveStaticFieldUsage(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODBODY:
                    //_mapModel.AddMethod(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODPARAMETER:
                    RemoveMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODPARAMETER);
                    break;
                case ReferenceType.METHODPARAMETERPUB:
                    RemovePublishedType(source.FullName, target.FullName, ReferenceType.METHODPARAMETERPUB);
                    break;
                case ReferenceType.METHODRETURNTYPE:
                    RemoveMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODRETURNTYPE);
                    break;
                case ReferenceType.METHODRETURNTYPEPUB:
                    RemovePublishedType(source.FullName, target.FullName, ReferenceType.METHODRETURNTYPEPUB);
                    break;
                case ReferenceType.STATICFIELD:
                    RemoveMemberField(source.FullName, target.FullName,t);
                    break;
                case ReferenceType.CONSTRUCTORINVOCATION:
                    RemoveConstructorInvocation(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODINVOCATION:
                    RemoveMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODINVOCATION);
                    break;
                case ReferenceType.STATICMETHODINVOCATION:
                    RemoveStaticMethodInvocation(source.FullName, target.FullName);
                    break;
                case ReferenceType.MEMBERPROPERTY:
                    RemoveMethodInvocation(source.FullName, target.FullName, ReferenceType.MEMBERPROPERTY);
                    break;
                default:
                    return;
            } 
        }
        private void AddMethodTypes(BaseClass c)
        {
            AddMethodTypes(c.FullName, c.ContainsStatic, c.ContainsNonStatic);
        }
        private void AddClassType(BaseClass c)
        {
            string type;
            if (c.IsInterface)
                type = "I";
            else if (c.IsAbstract)
                type = "A";
            else
                type = "C";

            AddClassType(c.FullName, type);
        }
        public void AddMethodSize(string className, int methodCount)
        {
            AddMethodSize(className, "" + methodCount);
        }
        private void AddMemberReferenceToMapModel(ReferenceObject source, ReferenceObject target, CodeReference r)
        {
            if (r.MemberProperty != null)
            {
                if (target.GetType().BaseType == typeof(BaseClass))
                {
                    BaseClass t = (BaseClass)target;
                    if (t.ContainsMember(r.MemberProperty))
                    {
                        AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODINVOCATION);
                        r.Type = ReferenceType.METHODINVOCATION;
                    }
                    else if(t.ContainsField(r.MemberProperty))
                    {
                        if (t.GetField(r.MemberProperty).IsStatic)
                        {
                            AddStaticFieldUsage(source.FullName, target.FullName);
                            r.Type = ReferenceType.STATICMEMBERREFERENCE;
                        }
                        else
                        {
                            AddMemberField(source.FullName, target.FullName,ReferenceType.MEMBERREFERENCE);
                            r.Type = ReferenceType.MEMBERREFERENCE;
                        }
                    }
                }
            }
            else
            {
                AddMemberField(source.FullName, target.FullName,ReferenceType.MEMBERFIELD);
            }
        }
        public void AddToMapModel(ReferenceObject source, ReferenceObject target, CodeReference r)
        {
            AddDependency(source.FullName, target.FullName,ReferenceType.DEPENDENCY);

            if (r.IsPublished)
            {
                if (r.Type == ReferenceType.METHODPARAMETER )
                    AddPublishedType(source.FullName, target.FullName, ReferenceType.METHODPARAMETERPUB);
                else if (r.Type == ReferenceType.METHODRETURNTYPE)
                    AddPublishedType(source.FullName, target.FullName, ReferenceType.METHODRETURNTYPEPUB);
                else if (r.Type == ReferenceType.MEMBERPROPERTY)
                    AddPublishedType(source.FullName, target.FullName, ReferenceType.MEMBERPROPERTY);
            }

            switch (r.Type)
            {
                case ReferenceType.BASE:
                    AddBaseType(source.FullName, target.FullName);
                    break;
                case ReferenceType.INTERFACE:
                    AddInterface(source.FullName, target.FullName);
                    break;
                case ReferenceType.MEMBERFIELD:
                    AddMemberField(source.FullName, target.FullName, ReferenceType.MEMBERFIELD);
                    break;
                case ReferenceType.MEMBERREFERENCE:
                    AddMemberReferenceToMapModel(source, target, r);
                    break;
                case ReferenceType.STATICMEMBERREFERENCE:
                    AddStaticFieldUsage(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODBODY:
                    //_mapModel.AddMethod(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODPARAMETER:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODPARAMETER);
                    break;
                case ReferenceType.METHODRETURNTYPE:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODRETURNTYPE);
                    break;
                case ReferenceType.STATICFIELD:
                    AddMemberField(source.FullName, target.FullName,ReferenceType.STATICFIELD);
                    break;
                case ReferenceType.CONSTRUCTORINVOCATION:
                    AddConstructorInvocation(source.FullName, target.FullName);
                    break;
                case ReferenceType.METHODINVOCATION:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.METHODINVOCATION);
                    break;
                case ReferenceType.STATICMETHODINVOCATION:
                    AddStaticMethodInvocation(source.FullName, target.FullName);
                    break;
                case ReferenceType.MEMBERPROPERTY:
                    AddMethodInvocation(source.FullName, target.FullName, ReferenceType.MEMBERPROPERTY);
                    break;
                default:
                    return;
            }

        }
        private void PrintActions()
        {
            Tracer.WriteTrace("Actions() -> {0}", _currentActions.AllActions.Count);
            Tracer.WriteTrace("Current Actions() -> {0}", _currentActions.CurrentActions.Count);
            foreach (GraphAction a in _currentActions.CurrentActions)
            {
                Tracer.WriteTrace("     {0}", GetAction(a));
            }
            Tracer.WriteTrace("Removed Actions() -> {0}", _currentActions.RemovedActions.Count);
            foreach (GraphAction a in _currentActions.RemovedActions)
            {
                Tracer.WriteTrace("     {0}", GetAction(a));
            }
        }
        private string GetAction(GraphAction a)
        {
            string s = a.Type + ": ";
            if (a.Node != null)
            {
                s += a.Node.Name;
                if (a.Edges != null)
                    s += ", Edges: " + a.Edges.Count;
            }
            else if (a.SingleEdge != null)
            {
                s += a.SingleEdge.Source.Name + " -> " + a.SingleEdge.Target.Name;
            }

            return s;
        }
        public void FinaliseClass(BaseClass c)
        {
            AddMethodTypes(c);
            AddClassType(c); 
            AddMethodSize(c.FullName, c.MethodCount);
        }
        public void ResetActions()
        {
            _currentActions = new GraphActions();
        }
        private void FireMapEvent(MapEventArgs e)
        {
            EventHandler<MapEventArgs> handler = ThrowMapChanged;
            if (handler != null)
                handler(this, e);
        }
    }
    public class MapEventArgs : EventArgs
    {
        private string _name, _reference;
        private ReferenceType _type;
        private bool _add, _pub;
        
        public MapEventArgs(string n,string r,ReferenceType t,bool b)
        {
            _name = n;
            _reference = r;
            _type = t;
            _add = b;
            _pub = false;
        }
        public MapEventArgs(string n, string r, ReferenceType t, bool b,bool p)
            : this(n,r,t,b)
        {
            _pub = p;
        }
        public string Name
        {
            get { return _name; }
        }
        public string Reference
        {
            get { return _reference; }
        }
        public ReferenceType Type
        {
            get { return _type; }
        }
        public bool Add
        {
            get { return _add; }
        }
        public bool Published
        {
            get { return _pub; }
        }
    }
}
