﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GraphController.Utility;
using System.ComponentModel;
using GraphController;
using GraphController.Common;
using GraphController.GUI;
using GraphController.Logic;
using ntnu.se.research.eades;
using melton.hayden.jepends.analysis;
using tosin.oyetoyan.research.support;
using tosin.oyetoyan.research.support.graph;
using java.util;

namespace GraphController
{
    public class MainController : INotifyPropertyChanged
    {
        private DependencyGraph _graphToVisualize, original, fileView;
        private MapModel  _mapModel, _fileViewMapModel, _originalMapModel, _previousMapModel, _lastResultMap;
        private GraphController.GUI.RefactoredEdges _reportEdges;
        private Dictionary<String, Namespace> _namespaceList;
        private Dictionary<String, BaseClass> _classList;
        private Dictionary<String, SourceFile> _sourceFileList;
        private Dictionary<String, Project> _projectList;
        private DependencyVertex _selectedVertex;
        private GraphActionType _selectedAction;
        private ObservableCollection<DependencyVertex> _classCollection, _currentCollection, _fileViewCollection;
        private ProgressbarForm _pf;
        private GraphLogic _currentLogic, _classLogic, _fileViewLogic;
        private SimulationResult _currentResult;
        private string _report, _lastSimulation, _programDir;
        private bool _initialized, _simulationFinished, _isClassView;

        public delegate void MapChangedCallback(object sender, MapEventArgs args);

        public MainController()
        {
            //CreateGraph();
            _pf = new ProgressbarForm();
            original = new DependencyGraph();
            fileView = new DependencyGraph();
            _graphToVisualize = original;
            _mapModel = new MapModel();
            _fileViewMapModel = new MapModel();
            _classLogic = new GraphLogic(_mapModel, original);
            _fileViewLogic = new GraphLogic(_fileViewMapModel, fileView);
            //_graphLayout = new MyGraphLayout();
            _classCollection = new ObservableCollection<DependencyVertex>();
            _fileViewCollection = new ObservableCollection<DependencyVertex>();
            _currentCollection = _classCollection;
            //graphLayout.Graph = _graphToVisualize;
            //_actions = new List<GraphAction>();
            //_removedActions = new List<GraphAction>();
            //_allActions = new List<GraphAction>();
            _namespaceList = new Dictionary<string, Namespace>();
            _classList = new Dictionary<string, BaseClass>();
            _sourceFileList = new Dictionary<string, SourceFile>();
            _projectList = new Dictionary<string, Project>();
            GUI.RefactoredEdges _reportEdges = new GUI.RefactoredEdges();
            _selectedAction = GraphActionType.MOVENODE;
            _initialized = false;
            _isClassView = false;
            _lastSimulation = "";
        }
        // REGION: Member property fields.
        #region MemberProperty region
        public bool IsInitialized
        {
            get { return _initialized; }
            set { _initialized = value; }
        }
        public bool IsClassView
        {
            get { return _isClassView; }
        }
        public DependencyVertex SelectedVertex
        {
            get { return _selectedVertex; }
            set { _selectedVertex = value; }
        }
        public ObservableCollection<DependencyVertex> ClassCollection
        {
            get { return _currentCollection; }
            set { _currentCollection = value; }
        }
        public Dictionary<String, BaseClass> ClassList
        {
            get { return _classList; }
            set { _classList = value; }
        }
        public ProgressbarForm ProgressBar
        {
            get { return _pf; }
        }
        //public MyGraphLayout GraphLayout
        //{
        //    get { return _graphLayout; }
        //    set { _graphLayout = value; }
        //}
        public void InitEventHandler()
        {
            _mapModel.ThrowMapChanged += new EventHandler<MapEventArgs>(OnMapChanged);
            _fileViewMapModel.ThrowMapChanged += new EventHandler<MapEventArgs>(OnMapChanged);
        }
        public DependencyGraph GraphToVisualize
        {
            get { return this._graphToVisualize; }
            set
            {
                if (!Equals(value, this._graphToVisualize))
                {
                    this._graphToVisualize = value;
                    _currentLogic.Graph = _graphToVisualize;
                    _currentLogic.AnalyseAndUpdateView();
                    NotifyPropertyChanged("GraphToVisualize");
                    //_graphLayout.ShouldLayout = false;
                }
            }
        }
        #endregion
        public Namespace GetNamespace(string ns)
        {
            return _namespaceList[ns];
        }

        public void ReLayout()
        {
            //_graphLayout.ShouldLayout = true;
            _currentLogic.Graph = _graphToVisualize;
            _currentLogic.AnalyseAndUpdateView();
            //_graphLayout.UpdateLayout();
            //_graphLayout.ShouldLayout = false;
            NotifyPropertyChanged("GraphToVisualize");
        }
        public void ShowResult()
        {
        }
        public DependencyGraph ChangeView()
        {
            if (_initialized)
            {
                if(!_isClassView)
                {
                    _isClassView = true; 
                    _currentLogic = _classLogic;
                    NotifyPropertyChanged("GraphToVisualize");
                    Tracer.WriteTrace("graph: {0}, Class: {1}, File: {2}", _graphToVisualize.EdgeCount, original.EdgeCount, fileView.EdgeCount);
                    return ReInitGraphAndDatagrid();
                }
                else
                {
                    _isClassView = false;
                    _graphToVisualize = fileView;
                    _currentLogic = _fileViewLogic;
                    _currentCollection = _fileViewCollection;
                    _currentLogic.AnalyseAndUpdateView();
                    NotifyPropertyChanged("GraphToVisualize");
                    Tracer.WriteTrace("graph: {0}, Class: {1}, File: {2}", _graphToVisualize.EdgeCount, original.EdgeCount, fileView.EdgeCount); 
                    return fileView;
                }
                
            }
            else
                return null;
        }
        public void SetSelectedAction(int i)
        {
            if (i > 0 && i < 4)
            {
                switch (i)
                {
                    case 1:
                        _selectedAction = GraphActionType.ADDEDGE;
                        break;
                    case 2:
                        _selectedAction = GraphActionType.REMOVEEDGE;
                        break;
                    case 3:
                        _selectedAction = GraphActionType.MOVENODE;
                        break;
                    default:
                        _selectedAction = GraphActionType.MOVENODE;
                        break;
                }
            }
        }
        public void SetMainClass(string s)
        {
            _currentLogic.MainClass = s;
            Tracer.WriteTrace("MainClass: {0}",_currentLogic.MainClass);
        }
        
        
        public bool IsActionMove()
        {
            return _selectedAction == GraphActionType.MOVENODE ? true : false;
        }
        private void AddEdgeToModel(Class source, Class target, CodeReference r)
        {
            _mapModel.AddToMapModel(source, target, r);
            _fileViewMapModel.AddToMapModel(source.SourceFile, target.SourceFile, r);
            //original.AddEdgeToGraph(source.Vertex, target.Vertex, r);
            //fileView.AddEdgeToGraph(source.SourceFile.Vertex, target.SourceFile.Vertex, r);
        }
        private void AddEdgeToModel(PartialClass source, Class target, CodeReference r)
        {
            _mapModel.AddToMapModel(source, target, r);
            //original.AddEdgeToGraph(source.Vertex, target.Vertex, r);
            try
            {
                //fileView.AddEdgeToGraph(r.SourceFile.Vertex, target.SourceFile.Vertex, r);
                _fileViewMapModel.AddToMapModel(r.SourceFile, target.SourceFile, r);
            }
            catch(NullReferenceException)
            {
                return;
            }
            
        }
        private void AddToMapModel(ReferenceObject s,ReferenceObject t, CodeReference r, bool flag)
        {
            if (flag)
                _mapModel.AddToMapModel(s, t, r);
            else
                _fileViewMapModel.AddToMapModel(s, t, r);
        }
        public void AddEdgeToModel(BaseClass source, BaseClass target, CodeReference r)
        {
            if (source.GetType() == typeof(PartialClass) && target.GetType() == typeof(Class))
                AddEdgeToModel((PartialClass)source, (Class)target, r);
            else if (source.GetType() == typeof(Class) && target.GetType() == typeof(Class))
                AddEdgeToModel((Class)source, (Class)target, r);
            else
            {
                //original.AddEdgeToGraph(source.Vertex, target.Vertex, r);
                AddToMapModel(source, target, r, true);
            }

        }
        #region GraphAction Methods
        public void UndoAction()
        {
            if (_isClassView)
                _mapModel.UndoAction();
            else
                _fileViewMapModel.UndoAction();
        }
        public void RedoAction()
        {
            if (_isClassView)
                _mapModel.RedoAction();
            else
                _fileViewMapModel.RedoAction();
        }
        private void RemoveEdgesInNode(GraphAction a)
        {
            //if (a.Edges == null)
            //{
            //    RemoveNodeFromMaps(a.Node);
            //    _graphToVisualize.RemoveVertex(a.Node);
            //    _currentCollection.Remove(a.Node);
            //}

            //else
            //{
            //    foreach (MyEdgeBase e in a.Edges)
            //    {
            //        _graphToVisualize.RemoveEdgeFromGraph(e);
            //        RemoveEdgeFromMaps(e);
            //    }
            //    RemoveNodeFromMaps(a.Node);
            //    _graphToVisualize.RemoveVertex(a.Node);
            //    _currentCollection.Remove(a.Node);
            //    Tracer.WriteTrace("RemoveNode{0} : return true", a.Node);
            //} 
        }

        public void PerformAddEdgeAction(DependencyVertex v)
        {
            GraphAction a = new GraphAction(GraphActionType.ADDEDGE, new DependencyEdge(_selectedVertex, v, ReferenceType.ADDEDINGUI));
            PerformAction(a, false);
            _selectedVertex = null;
        }

        public void PerformAddNodeAction(DependencyVertex v)
        {
            GraphAction a = new GraphAction(GraphActionType.ADDNODE, v);
            PerformAction(a, false);
        }
        public void PerformEditReferencesInMaps(DependencyEdge e, List<ReferenceType> remove, List<ReferenceType> add)
        {
            e.Types.Sort();
            remove.Sort();
            if (e.Types.SequenceEqual(remove) && add.Count <= 0)
            {
                PerformRemoveEdgeAction(e);
            }
            else
            {
                GraphAction a = new GraphAction(GraphActionType.EDITREFERENCES, e, remove, add);
                PerformAction(a, false);
            }
        }
        public void PerformRemoveNodeAction(DependencyVertex v)
        {
            GraphAction a = new GraphAction(GraphActionType.REMOVENODE, v);
            if (v != null)
            {
                DependencyVertex temp;
                if (!_graphToVisualize.ContainsVertex(v))
                {
                    temp = _graphToVisualize.GetVertex(v.FullName);
                }
                else
                    temp = v;
                if(temp != null)
                {
                    var list = _graphToVisualize.Edges.Where(n => n.Source == temp || n.Target == temp);
                    a.Edges = new List<DependencyEdge>();
                    foreach(DependencyEdge e in list)
                    {
                        a.Edges.Add(new DependencyEdge(e.Source,e.Target,e));
                    }
                    PerformAction(a, false);
                }
            }
        }
        private void PerformAction(GraphAction a, bool flag)
        {
            if(_isClassView)
            {
                _mapModel.PerformAction(a, flag);
            }
            else
            {
                _fileViewMapModel.PerformAction(a, flag);
            }
        }
        public void PerformRemoveEdgeAction(DependencyEdge edge)
        {
            GraphAction a = new GraphAction(GraphActionType.REMOVEEDGE, new DependencyEdge(edge.Source,edge.Target,edge));
            PerformAction(a, false);
            _selectedVertex = null;
        }
        private void AddEdgeClickAction(DependencyVertex data)
        {
            try
            {
                if (_selectedVertex != null)
                {
                    if (data.FullName == _selectedVertex.FullName)
                        return;
                    /*MessageBoxResult r = MessageBox.Show("You want to link " + _selectedVertex.FullName + " to " + data.Name,"Add Edge",MessageBoxButton.YesNo);
                    Tracer.WriteTrace("Result: {0}", r);
                    if (r == MessageBoxResult.Yes)
                    {*/
                    Tracer.WriteTrace("Linking {0} to {1}", _selectedVertex.FullName, data.FullName);
                    GraphAction a = new GraphAction(GraphActionType.ADDEDGE, new DependencyEdge(_selectedVertex, data, ReferenceType.ADDEDINGUI));
                    PerformAction(a, false);
                    _selectedVertex = null;
                    //}
                }
                else
                    _selectedVertex = data;


            }
            catch (ArgumentOutOfRangeException)
            {
                Tracer.WriteTrace("AddEdgeClick() -> ArgumentouofRange");
            }
            catch (Exception)
            {
                Tracer.WriteTrace("AddEdgeClick() -> Exception");
                _selectedVertex = null;
            }
        }
        private void RemoveEdgeClickAction(DependencyVertex data)
        {
            try
            {
                if (_selectedVertex != null)
                {
                    if (data.FullName == _selectedVertex.FullName)
                        return;
                    /*MessageBoxResult r = MessageBox.Show("You want to delete the edge between " + _selectedVertex.FullName + " and " + data.Name,"Remove Edge",MessageBoxButton.YesNo);
                    Tracer.WriteTrace("Result: {0}", r);
                    if (r == MessageBoxResult.Yes)
                    {*/
                    DependencyEdge edge;
                    if (_graphToVisualize.TryGetEdge(_selectedVertex, data, out edge))
                    {
                        Tracer.WriteTrace("Removing edge from {0} to {1}", _selectedVertex.FullName, data.FullName);
                        PerformRemoveEdgeAction(edge);
                    }

                    //}
                }
                else
                    _selectedVertex = data;


            }
            catch (Exception)
            {
                _selectedVertex = null;
            }
        }
        public void DoAction(DependencyVertex data)
        {
            if (_selectedAction == GraphActionType.ADDEDGE)
            {
                AddEdgeClickAction(data);
            }
            else
            {
                RemoveEdgeClickAction(data);
            }
        }
        
        #endregion
        #region Calculate Graph based on MapModel
        private DependencyGraph RecalculateMapModel(MapModel m)
        {
            DependencyGraph graph = new DependencyGraph();
            Dictionary<String, BaseClass> classes = new Dictionary<string, BaseClass>();
            object[] keyset = m.AllDependencies.keySet().toArray();
            object[] values = m.AllDependencies.values().toArray();

            for(int i = 0; i < keyset.Length; i++)
            {
                
                string s = (string)keyset[i];
                BaseClass cl, oldCl;
                if (!classes.TryGetValue(s, out cl))
                {
                    if (_classList.TryGetValue(s, out oldCl))
                    {
                        if (oldCl.GetType() == typeof(PartialClass))
                            cl = new PartialClass((PartialClass)oldCl);
                        else
                            cl = new Class((Class)oldCl);

                        classes.Add(cl.FullName, cl);
                    }
                    else
                    {
                        cl = new Class(s,"RefactorSourcefile");
                        classes.Add(cl.FullName, cl);
                        //_classList.Add(cl.FullName, cl);
                    }
                }
                
                if(!graph.ContainsVertex(cl.Vertex)) graph.AddVertex(cl.Vertex);
               
                object[] valueSet = ((Set)values[i]).toArray();

                for (int j = 0; j < valueSet.Length; j++)
                {
                    string dep = (string)valueSet[j];
                    BaseClass depClass, oldDep;
                    
                    if(!classes.TryGetValue(dep,out depClass))
                    {
                        if(_classList.TryGetValue(dep, out oldDep))
                        {
                            if (oldDep.GetType() == typeof(PartialClass))
                                depClass = new PartialClass((PartialClass)oldDep);
                            else
                                depClass = new Class((Class)oldDep);

                            classes.Add(depClass.FullName, depClass);
                        }
                        else
                        {
                            depClass = new Class(dep, "RefactorSourcefile");
                            classes.Add(depClass.FullName, depClass);
                            //_classList.Add(depClass.FullName, depClass);
                        }
                    }

                    if (!graph.ContainsVertex(depClass.Vertex))
                        graph.AddVertex(depClass.Vertex);
                    

                    graph.AddEdgeToGraph(new DependencyEdge(cl.Vertex, depClass.Vertex, GetReferencesFromMapModel(m,s, dep)));

                }
                
            }
            return graph;
        }
        private List<ReferenceType> GetReferencesFromMapModel(MapModel m,string c,string s)
        {
            List<ReferenceType> types = new List<ReferenceType>();

            Set set = (Set)m.MethodInvocations.get(c);
            if (set != null && set.contains(s))
                types.Add(ReferenceType.METHODINVOCATION);


            set = (Set)m.StaticMethodInvocations.get(c);
            if (set != null && set.contains(s))
                types.Add(ReferenceType.STATICMETHODINVOCATION);

            set = (Set)m.Memberfields.get(c);
            if (set != null && set.contains(s))
                types.Add(ReferenceType.MEMBERFIELD);


            set = (Set)m.Interfacetypes.get(c);
            if (set != null && set.contains(s))
                types.Add(ReferenceType.INTERFACE);

            set = (Set)m.Basetypes.get(c);
            if (set != null && set.contains(s))
                types.Add(ReferenceType.BASE);

            set = (Set)m.Constructorinvocations.get(c);
            if (set != null && set.contains(s))
                types.Add(ReferenceType.CONSTRUCTORINVOCATION);

            
            

            return types;
        }
        #endregion
        public void FinishedLoading()
        {
            DisposeProgressBar();
            _originalMapModel = new MapModel(_mapModel);
            _previousMapModel = new MapModel(_mapModel);
            _reportEdges = new GUI.RefactoredEdges();
            _graphToVisualize = original;
            _initialized = true;
            _isClassView = true;
        }
        public void DisposeProgressBar()
        {
            _pf.Dispose();
        }
        public DependencyGraph Rollback()
        {
            _lastResultMap = new MapModel(_mapModel);
            _mapModel = new MapModel(_originalMapModel);
            _currentResult = null;
            _reportEdges = new GUI.RefactoredEdges();
            //_currentLogic.Model = _mapModel;
            return ReInitGraphAndDatagrid();
        }
        public DependencyGraph RollbackPreviousResult()
        {
            _lastResultMap = new MapModel(_mapModel);
            _mapModel = new MapModel(_previousMapModel);
            return ReInitGraphAndDatagrid();
        }
        public void ReInit()
        {
            _classList = new Dictionary<String, BaseClass>();
            _projectList = new Dictionary<string, Project>();
            _sourceFileList = new Dictionary<string, SourceFile>();
            original = new DependencyGraph();
            fileView = new DependencyGraph();
            _graphToVisualize = original;
            _mapModel = new MapModel();
            _fileViewMapModel = new MapModel();
            _classLogic = new GraphLogic(_mapModel, original);
            _fileViewLogic = new GraphLogic(_fileViewMapModel, fileView);
            _currentLogic = _classLogic;
            _classCollection.Clear();
            _fileViewCollection.Clear();
            _pf = new ProgressbarForm();
        }
        public void WriteReport(string dir)
        {
            string path = dir + "\\SimulationData\\Classes";
            string filePath = dir + "\\SimulationData\\Files";

            _mapModel.WriteMapsToFiles(path);
            _fileViewMapModel.WriteMapsToFiles(filePath);
        }
        #region Graph Visualization methods
        public void AddSourceFile(SourceFile c)
        {
            //fileView.AddVertex(c.Vertex);
            _sourceFileList.Add(c.FullName, c);
            _fileViewMapModel.AddClass(c.FullName);
            _fileViewCollection.Add(c.Vertex);
        }
        public void AddClass(BaseClass c)
        {
            if (!_classList.ContainsKey(c.FullName))
            {
                if (c.FullName != null)
                {
                    //vertexMap[_currentNamespace].Add(v.Name, v);
                    _classList.Add(c.FullName, c);
                    c.Namespace.AddClass(c);
                    _mapModel.AddClass(c.FullName);
                    //original.AddVertex(c.Vertex);
                    //_namespaceList[_currentNamespace].Graph.AddVertexToGraph(c.Vertex);
                }
            }

        }
        /*public void HighlightVertex(MyVertexBase v)
        {
            if (_graphToVisualize.ContainsVertex(v))
            {
                _graphLayout.HighlightVertex(v, "Highlighted");
            }
        }*/
        private void AddVertex(MapEventArgs me,bool source)
        {
            if(source)
            {
                SourceFile s = null;
                _sourceFileList.TryGetValue(me.Name, out s);
                if (!fileView.ContainsVertex(s.Vertex))
                {
                    fileView.AddVertex(s.Vertex);
                    _fileViewCollection.Add(s.Vertex);
                }
            }
            else
            {
                BaseClass c = null;
                _classList.TryGetValue(me.Name, out c);
                if (!original.ContainsVertex(c.Vertex))
                {
                    original.AddVertex(c.Vertex);
                    _classCollection.Add(c.Vertex);
                }
            }
        }
        private void AddEdge(MapEventArgs me, bool source)
        {
            DependencyGraph temp;
            if (source)
            {
                temp = fileView;
            }
            else
            {
                temp = original;
            }
            if (me.Type == ReferenceType.DEPENDENCY)
                return;
            //    temp.AddEdge(me.Name, me.Reference, ReferenceType.USES, me.Published);
            //else
                temp.AddEdge(me.Name, me.Reference, me.Type, me.Published);
        }
        private void RemoveVertex(MapEventArgs me, bool source)
        {
            if (source)
            {
                SourceFile s = null;
                _sourceFileList.TryGetValue(me.Name, out s);
                if (fileView.ContainsVertex(s.Vertex))
                {
                    fileView.RemoveVertex(s.Vertex);
                    _fileViewCollection.Remove(s.Vertex);
                }
            }
            else
            {
                BaseClass c = null;
                _classList.TryGetValue(me.Name, out c);
                original.RemoveVertex(c.Vertex);
                _classCollection.Remove(c.Vertex);
            }
        }
        private void RemoveDependency(MapEventArgs me, bool source)
        {
            DependencyGraph temp;
            if (source)
            {
                temp = fileView;
            }
            else
            {
                temp = original;
            }
            
            temp.RemoveDependency(me.Name, me.Reference, me.Type);
        }
        private void MapEventHandler(MapEventArgs me, bool source)
        {
            if(me.Add)
            {
                switch (me.Type)
                {
                    case ReferenceType.NODE:
                        AddVertex(me, source);
                        break;
                    default:
                        AddEdge(me, source);
                        return;
                }
            }
            else
            {
                switch (me.Type)
                {
                    case ReferenceType.NODE:
                        RemoveVertex(me, source);
                        break;
                    default:
                        RemoveDependency(me, source);
                        return;
                }
            }
        }
        #endregion
        

        public bool ContainsClass(string fullname)
        {
            if (_classList.ContainsKey(fullname))
                return true;
            else return false;
        }
        public BaseClass GetClass(string fullname)
        {
            if (_classList.ContainsKey(fullname))
                return _classList[fullname];
            else
                return null;
        }
        public string GetEdgeInformation(DependencyVertex v)
        {
            return _graphToVisualize.GetEdgeInformationForVertex(v);
        }
        #region GraphLogic Encapsulation methods
        public void SetInputParameters(int topKFanin, int topKCrss, int topKScc, string mainClass)
        {
            _currentLogic.TopKFanin = topKFanin;
            _currentLogic.TopKCrss = topKCrss;
            _currentLogic.TopKScc = topKScc;
            if (mainClass == "" || mainClass == null)
                throw new ArgumentNullException();
            else
                _currentLogic.MainClass = mainClass;
        }
        public void SetTable1SortingPriority(string s)
        {
            _currentLogic.SetSortingPriority(s);
        }
        public int GetTopKFanin()
        {
            return _currentLogic.TopKFanin;
        }
        public int GetTopKCRSS()
        {
            return _currentLogic.TopKCrss;
        }
        public int GetTopKSCC()
        {
            return _currentLogic.TopKScc;
        }
        public string GetMainClass()
        {
            return _currentLogic.MainClass;
        }
        public SimulationResult GetCurrentResult()
        {
            return _currentResult;
        }
        public void SetNumOfRefactoring(int number)
        {
            _currentLogic.NumOfRefactoring = number;
        }
        public void SetReductionRate(double rate)
        {
            _currentLogic.ReductionRate = (int)rate;
        }
        public List<CycleData> GetCycleData()
        {
            return _currentLogic.CycleData;
        }
        public List<SingleComponentRefactoringData> CalculateSingleComponentCandidateData()
        {
            List<SingleComponentRefactoringData> data = new List<SingleComponentRefactoringData>();
            List s = _currentLogic.GetSingleComponentCandidateData(); // "source,target,crss,fanin,fanout,scc,reductionrate,decision"
            object[] objects = s.toArray();
            _lastSimulation = "Single Component Refactoring: \nComponents: " + s.size();

            foreach (object o in objects)
            {
                SingleComponentRefactoringData single;
                string line = (string)o;
                Tracer.WriteTrace("\tString from tool: {0}", line);
                string[] arr = line.Split(',');
                try
                {
                    string source = arr[0];
                    string target = arr[1];
                    int reach = Convert.ToInt32(arr[2]);
                    int fanin = Convert.ToInt32(arr[3]);
                    int fanout = Convert.ToInt32(arr[4]);
                    int scc = Convert.ToInt32(arr[5]);
                    double reduction = double.Parse(arr[6],CultureInfo.InvariantCulture);
                    string decision = arr[7];
                    single = new SingleComponentRefactoringData(source, target,fanin,fanout, reach, scc, reduction, decision);
                    data.Add(single);
                }
                catch (ArgumentNullException)
                {
                    continue;
                }
                catch (FormatException e)
                {
                    Tracer.WriteTrace("FormatException: {0}",e.StackTrace);
                    continue;
                }
                catch (OverflowException)
                {
                    continue;
                }

            }
            return data;
        }
        public List<SystemRefactoringData> CalculateSystemCandidateData(double rate)
        {
            List<SystemRefactoringData> data = new List<SystemRefactoringData>();
            List s = _currentLogic.GetSystemCandidateData(rate); //"class,crss,fanin,scc,reducedProp,Refactor_decision"
            object[] objects = s.toArray();
            _lastSimulation = "System Refactoring: \nComponents: " + s.size();
            _currentLogic.ReductionRate = rate;

            foreach (object o in objects)
            {
                SystemRefactoringData single;
                string line = (string)o;
                Tracer.WriteTrace("\tString from tool: {0}", line);
                string[] arr = line.Split(',');
                try
                {
                    string source = arr[0];
                    int reach = Convert.ToInt32(arr[1]);
                    int fanin = Convert.ToInt32(arr[2]);
                    int scc = Convert.ToInt32(arr[3]);
                    double reduction = double.Parse(arr[4], CultureInfo.InvariantCulture);
                    string decision = arr[5];
                    single = new SystemRefactoringData(source,reach,fanin,scc,reduction,decision);
                    data.Add(single);
                }
                catch (ArgumentNullException)
                {
                    continue;
                }
                catch (FormatException e)
                {
                    Tracer.WriteTrace("FormatException: {0}", e.StackTrace);
                    continue;
                }
                catch (OverflowException)
                {
                    continue;
                }

            }
            return data;
        }
        public SimulationResult SimulateCycleRefactoring(List<CycleRefactoringData> data)
        {
            _simulationFinished = true;
            
            _currentResult = _currentLogic.SimulateCycleRefactoring(data);
            return _currentResult;// = _logic.SimulateCycleRefactoring(data);
        }
        public SimulationResult SimulateSingleComponentRefactoring(List<SingleComponentRefactoringData> data)
        {
            _simulationFinished = true;
            _currentResult = _currentLogic.SimulateSingleComponentRefactoring(data);
            return _currentResult;
        }
        public SimulationResult SimulateSystemRefactoring(int number)
        {
            _simulationFinished = true;
            _currentResult = _currentLogic.SimulateSystemRefactoring(number);
            return _currentResult;
        }
        private void GenerateSubGraph()
        {
            List l = _currentResult.RefactoredEdges;
            Tracer.WriteTrace("Refactored Edges: {0}", l.size());
            StringBuilder sb = new StringBuilder();
            sb.Append("Report: \n");
            sb.Append(_lastSimulation + "\n\n");
            for(int i = 0; i < l.size(); i++)
            {
                string o = (string)l.get(i).ToString();
                sb.Append(o + "\n");
                Tracer.WriteTrace("\tindex {0}: {1}", i, o);
            }
            _report = sb.ToString();
        }
        public GraphController.GUI.RefactoredEdges GetReport()
        {
            if (_reportEdges.Size == 0)
                return null;

            //GenerateSubGraph();
            return _reportEdges;
        }
        public List<CycleRefactoringData> CalculateCycleCandidateData(CycleData d,bool flag)
        {
            List<CycleRefactoringData> data = new List<CycleRefactoringData>();
            d.PrindCycleSet();
            List s = _currentLogic.GetCycleCandidates(d.CycleSet, flag); //"from,to,crss,scc,reducedRate,Refactor_decision" 
            object[] objects = s.toArray();
            _lastSimulation = "Cycle Refactoring: \nSize: " + d.Size + "\tMain Class: " + d.MainClass.FullName;

            Tracer.WriteTrace("Calculate Cycle: {0}", objects.Count());
            foreach(object o in objects)
            {
                CycleRefactoringData cycle;
                string line = (string)o;
                Tracer.WriteTrace("Calculate Cycle: line: {0}", line);
                string[] arr = line.Split(',');
                try
                {
                    string source = arr[0];
                    string target = arr[1];
                    int reach = Convert.ToInt32(arr[2]);
                    int scc = Convert.ToInt32(arr[3]);
                    double reduction = double.Parse(arr[4], CultureInfo.InvariantCulture);
                    string decision = arr[5];
                    cycle = new CycleRefactoringData(source,target,reach,scc,reduction,decision);
                    data.Add(cycle);
                }
                catch(ArgumentNullException)
                {
                    continue;
                }
                catch(FormatException)
                {
                    continue;
                }
                catch(OverflowException)
                {
                    continue;
                }
                
            }
            Tracer.WriteTrace("Returning with {0} cycleobjects",data.Count);
            return data;
        }
        #endregion 
        
        #region MapModel insertion methods

        
        
        #endregion
        public void FinaliseClass(BaseClass c)
        {
            c.EdgesCalculated();
            _mapModel.FinaliseClass(c);
        }
        public SimulationResult GetFitness()
        {
            return _currentLogic.GetFitness();
        }
        public void AddResult(SimulationResult r)
        {
            if (r.WasAdded)
                return;

            if (_lastResultMap != null)
                _mapModel = new MapModel(_lastResultMap);

            _previousMapModel = new MapModel(_mapModel);
            _reportEdges.AddReport(r.RefactoredEdges);
            Tracer.WriteTrace("Refactored Edges: ");
            for (int i = 0; i < r.RefactoredEdges.size(); i++ )
            {
                Tracer.WriteTrace("\tIndex({0}): {1}", i, r.RefactoredEdges.get(i));
            }
            _mapModel.ResetActions();
            r.WasAdded = true;
        }
        
        public DependencyGraph Recalculate()
        {
            //_graphToVisualize.Analyse();
            if(_initialized && _isClassView)
            {
                return ReInitGraphAndDatagrid();
            }
            return null;
        }
        private DependencyGraph ReInitGraphAndDatagrid()
        {
            original = RecalculateMapModel(_mapModel);
            _graphToVisualize = original;
            RecalculateClassCollection();
            _currentCollection = _classCollection;
            _classLogic = new GraphLogic(_mapModel, _graphToVisualize,_currentLogic);
            _currentLogic = _classLogic;
            //_currentLogic.Graph = original;
            _currentLogic.AnalyseAndUpdateView();
            foreach(CycleData d in GetCycleData())
            {
                d.PrindCycleSet();
            }
            return _graphToVisualize;
        }
        public ObservableCollection<DependencyVertex> RecalculateClassCollection()
        {
            ObservableCollection<DependencyVertex> coll = new ObservableCollection<DependencyVertex>();
            foreach(DependencyVertex v in _graphToVisualize.Vertices)
            {
                coll.Add(v);
            }
            _classCollection = coll;
            return coll;
        }
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }

        private void OnMapChanged(object sender, EventArgs e)
        {
            //if (this.InvokeRequired)
            //{
            //    this.BeginInvoke(new MapChangedCallback(OnMapChanged), new object[] { sender, e });
            //}
            //else
            //{
            //    frmcomm.CommTextBox("Receiver message: " + e.DiaString + "\r\n", Color.Red);
            //}
            if(sender.GetType() == typeof(MapModel) && e.GetType() == typeof(MapEventArgs))
            {
                MapModel m = (MapModel)sender;
                MapEventArgs me = (MapEventArgs)e;
                if(m == _mapModel)
                {
                    MapEventHandler(me, false);   
                }
                else
                {
                    MapEventHandler(me, true);  
                }
                Tracer.WriteTrace("Mapevent: {0}, {1}, {2}", me.Type, me.Name, me.Add);
            }

        }

        public void AddNodeFromGUI(string name,string namesp)
        {
            Namespace ns = new Namespace(namesp);
            Class c = new Class(name, ns, new SourceFile("AddedInGUI"), "Unknown");
            _classList.Add(c.FullName, c);
            PerformAddNodeAction(c.Vertex);
        }
    }
}
