﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GraphController.Common;
using tosin.oyetoyan.research.support.fitness;
using tosin.oyetoyan.research.support.graph;
using java.util;

namespace GraphController.GUI
{
    public class RefactoringData
    {
        private double _reductionRate;
        private string _originalDecision;
        private RefactoringStrategy _strategy;

        public RefactoringData()
        {

        }
        public RefactoringData(double rate,string strat)
        {
            _reductionRate = rate;
            _originalDecision = strat;
            switch (strat)
            {
                case "SFC":
                    _strategy = RefactoringStrategy.STATICFINALFIELDCOPY;
                    break;
                case "DI":
                    _strategy = RefactoringStrategy.EXTRACTINTERFACE;
                    break;
                case "SR":
                    _strategy = RefactoringStrategy.SINLGETONREGISTRY;
                    break;
                case "PSR":
                    _strategy = RefactoringStrategy.PROTOTYPEREGISTRY;
                    break;
                case "PR":
                    _strategy = RefactoringStrategy.PROTOTYPEREGISTRY;
                    break;
                case "SMI":
                    _strategy = RefactoringStrategy.STATICMETHODINLINING;
                    break;
                case "NONE":
                    _strategy = RefactoringStrategy.NONE;
                    break;
                case "NODECISION":
                    _strategy = RefactoringStrategy.NONE;
                    break;
                default:
                    _strategy = RefactoringStrategy.UNKNOWN;
                    break;
            }
        }
        public double ReductionRate
        {
            get { return _reductionRate; }
            set { _reductionRate = value; }
        }
        public RefactoringStrategy Strategy
        {
            get { return _strategy; }
            set { _strategy = value; }
        }
        public string GetOriginalDecision()
        {
            return _originalDecision;
        }
    }
    public class SystemRefactoringData : RefactoringData
    {
        private string _source;
        private int _reachability, _fanin, _scc;

        public SystemRefactoringData(string c,int reachability,int fanin,int scc,double red,string dec)
            : base(red,dec)
        {
            _source = c;
            _reachability = reachability;
            _fanin = fanin;
            _scc = scc;
        }
        public string Class
        {
            get { return _source; }
        }
        public int Reachability
        {
            get { return _reachability; }
        }
        public int Fanin
        {
            get { return _fanin; }
        }
        public int SCC
        {
            get { return _scc; }
        }
    }
    public class CycleRefactoringData : RefactoringData
    {
        private string _source, _target;
        private int _reachability, _scc;

        public CycleRefactoringData(string s,string t,int r, int scc,double red,string dec)
            : base(red,dec)
        {
            _source = s;
            _target = t;
            _reachability = r;
            _scc = scc;
        }
        public String Source
        {
            get { return _source; }
        }
        public String Target
        {
            get { return _target; }
        }
        public int SCC
        {
            get { return _scc; }
        }
        public int Reachability
        {
            get { return _reachability; }
        }
    }
    public class SingleComponentRefactoringData : RefactoringData
    {
        private string _source, _target;
        private int _fanin, _fanout,_reachability, _scc;
        private bool _isSelected;
        //"source,target,crss,fanout,scc,reductionrate,decision" 
        public SingleComponentRefactoringData(string source,string target,int fanin,int fanout, int reach, int scc,double red,string dec)
               : base(red,dec)
        {
            _source = source;
            _target = target;
            _fanin = fanin;
            _fanout = fanout;
            _isSelected = false;
            _reachability = reach;
            _scc = scc;
        }

        public string Source
        {
            get{ return _source; } 
        }
        public string Target
        {
            get{ return _target; }  
        }
        public int Reachability
        {
            get { return _reachability; }
        }
        public int FanIn
        {
            get { return _fanin; }
        }
        public int FanOut
        {
            get { return _fanout; }
        }
        public int SCC
        {
            get { return _scc; }
        }
        public bool IsSelected
        {
            get { return _isSelected; }
            set { _isSelected = value; }
        }
    }
    public enum RefactoringStrategy
    {
        STATICFINALFIELDCOPY, EXTRACTINTERFACE, INJECTEXTRACTINTERFACE, STATICMETHODINLINING, SINLGETONREGISTRY, PROTOTYPEREGISTRY, NONE, UNKNOWN
    }

    public class SimulationResult
    {
        private int _numberOfRefactorings;
        private string _info;
        private FitnessValue _fitness;
        private List _refactoredEdges, _finalCandidates;
        private Map _refactoringReport;
        private bool _wasAdded;


        public SimulationResult()
        {
            _numberOfRefactorings = 0;
            _fitness = new FitnessValuesOthers();
            _refactoredEdges = new ArrayList();
            _finalCandidates = new ArrayList();
            _refactoringReport = new HashMap();
            _wasAdded = false;
        }
        public SimulationResult(string info,int numRefac, FitnessValue f, List edges, List cand, Map report)
        {
            _numberOfRefactorings = numRefac;
            _fitness = f;
            _refactoredEdges = edges;
            _finalCandidates = cand;
            _refactoringReport = report;
        }
        public int NumberOfRefactorings
        {
            get { return _numberOfRefactorings; }
            set { _numberOfRefactorings = value; }
        }
        public string Info
        {
            get { return _info; }
            set { _info = value; }
        }
        public FitnessValue Fitness
        {
            get { return _fitness; }
            set { _fitness = value; }
        }
        public bool WasAdded
        {
            get { return _wasAdded; }
            set { _wasAdded = value; }
        }
        public List FinalCandidates
        {
            get { return _finalCandidates; }
            set { _finalCandidates = value; }
        }
        public List RefactoredEdges
        {
            get { return _refactoredEdges; }
            set { _refactoredEdges = value; }
        }
        public Map RefactoringReport
        {
            get { return _refactoringReport; }
            set { _refactoringReport = value; }
        }
        public int GetPreHistogramSize()
        {
            return _fitness.getPreRefactoring_Histogram().size();
        }
        public string GetRangeInHistogram(bool pre, int index)
        {
            if (pre)
            {
                Histogram.Hist h = (Histogram.Hist)_fitness.getPreRefactoring_Histogram().get(index);
                return h.getRange();
            }
            else
            {
                Histogram.Hist h = (Histogram.Hist)_fitness.getPostRefactoring_Histogram().get(index);
                return h.getRange();
            }
        }
        public int GetFrequencyInHistogram(bool pre, int index)
        {
            try
            {
                if (pre)
                {
                    Histogram.Hist h = (Histogram.Hist)_fitness.getPreRefactoring_Histogram().get(index);
                    return h.getFreq();
                }
                else
                {
                    Histogram.Hist h = (Histogram.Hist)_fitness.getPostRefactoring_Histogram().get(index);
                    return h.getFreq();
                }
            }
            catch(Exception)
            {
                return -1;
            }
        }
        public object[] PostStdDev
        {
            get { return _fitness.getPostRefactoring_StdDev().toArray(); }
        }
        public double Pre_Mean
        {
            get { return _fitness.getPreRefactoring_Mean(); }
        }
        public object[] Post_Mean
        {
            get { return _fitness.getPostRefactoring_Mean().toArray(); }
        }
        public double Pre_StdDev
        {
            get { return _fitness.getPreRefactoring_StdDev(); }
        }
        public int Pre_MaxSCC
        {
            get { return _fitness.getPreRefactoring_MaxSCCSize(); }
        }
        public int Post_MaxSCC
        {
            get { return _fitness.getPostRefactoring_MaxSCCSize(); }
        }
        public int Pre_NumberOfEdges
        {
            get { return _fitness.getPreRefactoringNumberOfEdges(); }
        }
        public int Post_NumberOfEdges
        {
            get { return _fitness.getPostRefactoringNumberOfEdges(); }
        }
        public int Pre_NumberOfNodes
        {
            get { return _fitness.getPreRefactoringNumberOfComponents(); }
        }
        public int Post_NumberOfNodes
        {
            get { return _fitness.getPostRefactoringNumberOfComponents(); }
        }

    }
    public class RefactoredEdges
    {
        private List _refactoredEdges;
        private List<RefactoredEdge> _edges;

        public RefactoredEdges()
        {
            _refactoredEdges = new ArrayList();
            _edges = new List<RefactoredEdge>();
        }
        private void InitEdges()
        {
            _edges.Clear();
            for(int i = 0; i < _refactoredEdges.size(); i++)
            {
                string line = (string)_refactoredEdges.get(i).ToString();
                string[] arr = line.Split('\t');
                RefactoredEdge edge = new RefactoredEdge(arr[0], arr[1], arr[2],arr[3]);
                _edges.Add(edge);
            }
        }
        public void AddReport(List edges)
        {
            for(int i = 0; i < edges.size(); i++)
            {
                if(!_refactoredEdges.contains(edges.get(i)))
                    _refactoredEdges.add(edges.get(i));
            }
            InitEdges();
        }
        public List<RefactoredEdge> Edges
        {
            get { return _edges; }
            set { _edges = value; }
        }
        
        public int Size
        {
            get { return _refactoredEdges.size(); }
        }
    }
    public class RefactoredEdge
    {
        private string _source, _target, _action, _strategy;

        public RefactoredEdge(string s, string t, string a, string e)
        {
            _source = s;
            _target = t;
            _action = a;
            _strategy = e;
        }
        public string Target
        {
            get { return _target; }
        }

        public string Source
        {
            get { return _source; }
        }
        public string Action
        {
            get { return _action; }
        }
        public string Strategy
        {
            get { return _strategy; }
        }
    }
}
