﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GraphController.Common;

namespace GraphController.Logic
{
    public class GraphActions
    {
        private List<GraphAction> _currentActions, _allActions, _removedActions;

        public GraphActions()
        {
            _currentActions = new List<GraphAction>();
            _allActions = new List<GraphAction>();
            _removedActions = new List<GraphAction>();
        }
        public List<GraphAction> RemovedActions
        {
            get { return _removedActions; }
            set { _removedActions = value; }
        }

        public List<GraphAction> AllActions
        {
            get { return _allActions; }
            set { _allActions = value; }
        }

        public List<GraphAction> CurrentActions
        {
            get { return _currentActions; }
            set { _currentActions = value; }
        }
        
        public void AddAction(GraphAction a)
        {
            _allActions.Add(a);
        }
        public void AddRemovedAction(GraphAction a)
        {
            _removedActions.Add(a);
            _currentActions.Remove(a);
        }
        public void RedoAction(GraphAction a)
        {
            _removedActions.Remove(a);
            //_currentActions.Add(a);
        }
        public void AddNewAction(GraphAction a)
        {
            _allActions.Add(a);
        }
        public void AddCurrentAction(GraphAction a)
        {
            _currentActions.Add(a);
        }
        public GraphAction GetLastCurrentAction()
        {
            return _currentActions.Last();
        }
        public GraphAction GetFirstRemovedAction()
        {
            foreach (GraphAction a in _allActions)
            {
                if (_removedActions.Contains(a))
                    return a;
            }
            return null;
        }
        public void SetGraphActions(GraphActions a)
        {
            _currentActions = a.CurrentActions;
            _allActions = a.AllActions;
            _removedActions = a.RemovedActions;
        }
    }
    public class GraphAction
    {
        private GraphActionType _type;
        private DependencyVertex _node;
        private List<DependencyEdge> _edges;
        private DependencyEdge _singleEdge;
        private List<ReferenceType> _toRemove, _toAdd;

        public GraphAction(GraphActionType t, DependencyVertex n)
        {
            _type = t;
            _node = n;
        }
        public GraphAction(GraphActionType t, DependencyEdge e)
        {
            _type = t;
            _singleEdge = new DependencyEdge(e.Source,e.Target, e);
        }
        public GraphAction(GraphActionType t, DependencyVertex n, List<DependencyEdge> l)
        {
            _type = t;
            _node = n;
            _edges = new List<DependencyEdge>();
            foreach(DependencyEdge e in l)
            {
                _edges.Add(new DependencyEdge(e.Source, e.Target, e));
            }
        }
        public GraphAction(GraphActionType t, DependencyEdge e, List<ReferenceType> l, List<ReferenceType> l2)
        {
            _type = t;
            _singleEdge = new DependencyEdge(e.Source, e.Target, e);
            _toRemove = l;
            _toAdd = l2;
        }
        public GraphAction()
        {
            _edges = new List<DependencyEdge>();
        }
        public GraphActionType Type
        {
            get { return _type; }
            set { _type = value; }
        }

        public DependencyVertex Node
        {
            get { return _node; }
            set { _node = value; }
        }

        public List<DependencyEdge> Edges
        {
            get { return _edges; }
            set { _edges = value; }
        }
        public DependencyEdge SingleEdge
        {
            get { return _singleEdge; }
        }
        public List<ReferenceType> ToRemove
        {
            get { return _toRemove; }
            set { _toRemove = value; }
        }
        public List<ReferenceType> ToAdd
        {
            get { return _toAdd; }
            set { _toAdd = value; }
        }
    }

    /// <summary>
    /// Private enum for use with GraphAction
    /// </summary>
    public enum GraphActionType
    {
        ADDNODE, REMOVENODE, ADDEDGE, REMOVEEDGE, MOVENODE, EDITREFERENCES
    }
}
