#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

namespace GraphController.Utility
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Text;
    using System.Globalization;

    /// <summary>
    /// Tracer class
    /// </summary>
    public static class Tracer
    {
        /// <summary>
        /// The trace switch used
        /// </summary>
        private static readonly TraceSwitch traceSwitch = new TraceSwitch("TraceLevelSwitch", "Entire application");

        /// <summary>
        /// Output specified logMessage if allowed by app.config
        /// </summary>
        /// <param name="logMessage">The logMessage format.</param>
        /// <param name="args">The parameters for the log message.</param>
        public static void Debug(string logMessage, params object[] args)
        {
            Trace.WriteIf(traceSwitch.TraceVerbose, DateTime.Now.ToString("o", CultureInfo.InvariantCulture) + " ");
            Trace.WriteLineIf(traceSwitch.TraceVerbose, string.Format(CultureInfo.InvariantCulture, logMessage, args));
        }

        /// <summary>
        /// Output specified logMessage if allowed by app.config
        /// </summary>
        /// <param name="logMessage">The logMessage format.</param>
        /// <param name="args">The parameters for the log message.</param>
        public static void Info(string logMessage, params object[] args)
        {
            Trace.WriteIf(traceSwitch.TraceInfo, DateTime.Now.ToString("o", CultureInfo.InvariantCulture) + " ");
            Trace.WriteLineIf(traceSwitch.TraceInfo, string.Format(CultureInfo.InvariantCulture, logMessage, args));
        }

        /// <summary>
        /// Output specified logMessage if allowed by app.config
        /// </summary>
        /// <param name="logMessage">The logMessage format.</param>
        /// <param name="args">The parameters for the log message.</param>
        public static void Warning(string logMessage, params object[] args)
        {
            Trace.WriteIf(traceSwitch.TraceWarning, DateTime.Now.ToString("o", CultureInfo.InvariantCulture) + " ");
            Trace.WriteLineIf(traceSwitch.TraceWarning, string.Format(CultureInfo.InvariantCulture, logMessage, args));
        }

        /// <summary>
        /// Output specified logMessage if allowed by app.config
        /// </summary>
        /// <param name="logMessage">The logMessage format.</param>
        /// <param name="args">The parameters for the log message.</param>
        public static void Error(string logMessage, params object[] args)
        {
            Trace.WriteIf(traceSwitch.TraceError, DateTime.Now.ToString("o", CultureInfo.InvariantCulture) + " ");
            Trace.WriteLineIf(traceSwitch.TraceError, string.Format(CultureInfo.InvariantCulture, logMessage, args));
        }
        public static void WriteTrace(string s,params object[] args)
        {
            Trace.WriteLine(string.Format(CultureInfo.CurrentCulture, s, args));
        }
        /// <summary>
        /// Indents this instance.
        /// </summary>
        public static void Indent()
        {
            Trace.Indent();
        }

        /// <summary>
        /// Dedents this instance.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Dedent", Justification = "This is the correct spelling")]
        public static void Dedent()
        {
            Trace.Unindent();
        }
    }
}
