﻿#region License
/*
Copyright 2014 Christian Thurmann-Nielsen

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#endregion

using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Threading.Tasks;
using EnvDTE;
using EnvDTE80;
using GraphController.Utility;

namespace Parser
{

    public class ClassFileFinder
    {
        private List<ProjectItem> _projectItems;
        private Projects _projects;
        //private string _programDir;

        public ClassFileFinder(Projects projects)
        {
            _projects = projects;
            _projectItems = new List<ProjectItem>();
        }

        private List<Project> GetProjects()
        {
            List<Project> list = new List<Project>();
            var item = _projects.GetEnumerator();
            while (item.MoveNext())
            {
                var project = item.Current as Project;
                if (project == null)
                {
                    continue;
                }

                if (project.Kind == ProjectKinds.vsProjectKindSolutionFolder)
                {
                    list.AddRange(GetSolutionFolderProjects(project));
                }
                else
                {
                    list.Add(project);
                }
            }

            return list;
        }

        private static List<Project> GetSolutionFolderProjects(Project solutionFolder)
        {
            List<Project> list = new List<Project>();


            for (var i = 1; i <= solutionFolder.ProjectItems.Count; i++)
            {
                var subProject = solutionFolder.ProjectItems.Item(i).SubProject;
                if (subProject == null)
                {
                    continue;
                }

                // If this is another solution folder, do a recursive call, otherwise add
                if (subProject.Kind == ProjectKinds.vsProjectKindSolutionFolder)
                {
                    list.AddRange(GetSolutionFolderProjects(subProject));
                }
                else
                {
                    list.Add(subProject);
                }
            }

            return list;
        }
        private ProjectItem GetProjectItems(ProjectItem item)
        {
            //base case
            if (item.ProjectItems == null || item.ProjectItems.Count == 0)
            {
                return item;
            }

            var items = item.ProjectItems.GetEnumerator();
            while (items.MoveNext())
            {
                var currentItem = (ProjectItem)items.Current;
                _projectItems.Add(GetProjectItems(currentItem));
            }

            return item;
        }

        public List<ClassFile> GetClassFiles()
        {
            IList<Project> projects;
            projects = GetProjects();
            List<ClassFile> classes = new List<ClassFile>();

            foreach (Project p in projects)
            {
                var items = p.ProjectItems.GetEnumerator();
                while (items.MoveNext())
                {
                    var item = (EnvDTE.ProjectItem)items.Current;
                    //Tracer.WriteTrace("\tFound item in project: {0}", item.Name);
                    //Recursion to get all ProjectItems
                    _projectItems.Add(GetProjectItems(item));
                }
            }


            var list = _projectItems.FindAll(n => n.Name.EndsWith(".cs"));
            Tracer.WriteTrace("Classfiles in solution: {0}", list.Count);
            foreach (ProjectItem i in list)
            {
                try
                {
                    //Tracer.WriteTrace("Classfile name: {0}", i.Name);
                    //Tracer.WriteTrace("Classfile path: {0}", i.Properties.Item("FullPath").Value);
                    //Tracer.WriteTrace("Classfile: project: {0}", i.ContainingProject.Name);
                    ClassFile f = new ClassFile(i.Name, i.Properties.Item("FullPath").Value, i.ContainingProject.Name);
                    classes.Add(f);
                    Tracer.WriteTrace("Found projectitem: {0}, {1}, {2}", i.Name, i.Properties.Item("FullPath").Value, i.ContainingProject.Name);
                }
                catch (System.NullReferenceException)
                {
                    Tracer.WriteTrace("Projectitem nullreference: {0}", i.Name);
                    continue;
                }


            }
            return classes;
        }

    }
    public class ClassFile
    {
        string _name, _path, _project;

        public ClassFile(string name, string path, string project)
        {
            _name = name;
            _path = path;
            _project = project;
        }
        public string Name
        {
            get { return _name; }
        }

        public string Path
        {
            get { return _path; }
        }

        public string Project
        {
            get { return _project; }
        }

    }
}